@@ -28,15 +28,13 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
-from typing import Optional
 from typing import TYPE_CHECKING
-from typing import Union
 
 import collections
 import copy
 import functools
 import hashlib
+import importlib.metadata
 import inspect
 import json
 import logging
@@ -51,6 +49,7 @@ import unicodedata
 import weakref
 import webbrowser
 from collections import defaultdict
+from collections.abc import Callable
 from datetime import datetime
 from datetime import timedelta
 from functools import wraps
@@ -60,11 +59,14 @@ from urllib.parse import unquote
 from urllib.parse import urlparse
 
 import precis_i18n.codec  # noqa: F401
+import qrcode
 from cryptography import x509
 from cryptography.hazmat.backends import default_backend
+from gi.repository import GdkPixbuf
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import GObject
+from gi.repository import Soup
 from nbxmpp.const import Affiliation
 from nbxmpp.const import Chatstate
 from nbxmpp.const import ConnectionProtocol
@@ -77,7 +79,9 @@ from nbxmpp.protocol import Iq
 from nbxmpp.protocol import JID
 from nbxmpp.structs import CommonError
 from nbxmpp.structs import ProxyData
+from packaging.requirements import Requirement
 from packaging.version import Version as V
+from qrcode.image.pil import PilImage as QrcPilImage
 
 from gajim.common import app
 from gajim.common import configpaths
@@ -85,6 +89,7 @@ from gajim.common import iana
 from gajim.common import types
 from gajim.common.const import CONSONANTS
 from gajim.common.const import GIO_TLS_ERRORS
+from gajim.common.const import NONREGISTERED_URI_SCHEMES
 from gajim.common.const import SHOW_LIST
 from gajim.common.const import SHOW_STRING
 from gajim.common.const import SHOW_STRING_MNEMONIC
@@ -157,7 +162,7 @@ def ascii_to_idn(host: str) -> str:
     return '.'.join(converted_labels)
 
 
-def parse_resource(resource: str) -> Optional[str]:
+def parse_resource(resource: str) -> str | None:
     '''
     Perform stringprep on resource and return it
     '''
@@ -192,7 +197,7 @@ def get_uf_sub(sub: str) -> str:
     return p_('Contact subscription', 'Unknown')
 
 
-def get_uf_ask(ask: Union[str, None]) -> str:
+def get_uf_ask(ask: str | None) -> str:
     if ask is None:
         return p_('Contact subscription', 'None')
 
@@ -202,7 +207,7 @@ def get_uf_ask(ask: Union[str, None]) -> str:
     return ask
 
 
-def get_uf_role(role: Union[Role, str], plural: bool = False) -> str:
+def get_uf_role(role: Role | str, plural: bool = False) -> str:
     ''' plural determines if you get Moderators or Moderator'''
     if not isinstance(role, str):
         role = role.value
@@ -224,7 +229,7 @@ def get_uf_role(role: Union[Role, str], plural: bool = False) -> str:
     return ''
 
 
-def get_uf_affiliation(affiliation: Union[Affiliation, str],
+def get_uf_affiliation(affiliation: Affiliation | str,
                        plural: bool = False
                        ) -> str:
     '''Get a nice and translated affilition for muc'''
@@ -248,12 +253,11 @@ def get_uf_affiliation(affiliation: Union[Affiliation, str],
     return ''
 
 
-def get_uf_relative_time(timestamp: float,
-                         timenow: Optional[float] = None) -> str:
-    date_time = datetime.fromtimestamp(timestamp)
-    if timenow:  # used by unittest
-        now = datetime.fromtimestamp(timenow)
-    else:
+def get_uf_relative_time(date_time: datetime,
+                         now: datetime | None = None
+                         ) -> str:
+
+    if now is None:  # used by unittest
         now = datetime.now()
     timespan = now - date_time
 
@@ -261,11 +265,11 @@ def get_uf_relative_time(timestamp: float,
         return _('Just now')
     if timespan < timedelta(minutes=15):
         minutes = int(timespan.seconds / 60)
-        return ngettext('%i min ago',
-                        '%i mins ago',
+        return ngettext('%s min ago',
+                        '%s mins ago',
                         minutes,
-                        minutes,
-                        minutes)
+                        str(minutes),
+                        str(minutes))
     today = now.date()
     if date_time.date() == today:
         format_string = app.settings.get('time_format')
@@ -295,7 +299,7 @@ def from_one_line(msg: str) -> str:
     return msg.replace('\\\\', '\\')
 
 
-def chatstate_to_string(chatstate: Optional[Chatstate]) -> str:
+def chatstate_to_string(chatstate: Chatstate | None) -> str:
     if chatstate is None:
         return ''
 
@@ -364,8 +368,23 @@ def get_contact_dict_for_account(account: str) -> dict[str, types.BareContact]:
     return contacts_dict
 
 
+def generate_qr_code(content: str) -> GdkPixbuf.Pixbuf | None:
+    qr = qrcode.QRCode(version=None,
+                       error_correction=qrcode.constants.ERROR_CORRECT_L,
+                       box_size=6,
+                       border=4)
+    qr.add_data(content)
+    qr.make(fit=True)
+
+    img = qr.make_image(image_factory=QrcPilImage).convert('RGB')
+    return GdkPixbuf.Pixbuf.new_from_bytes(
+        GLib.Bytes.new(img.tobytes()),
+        GdkPixbuf.Colorspace.RGB, False, 8,
+        img.width, img.height, img.width*3)
+
+
 def play_sound(sound_event: str,
-               account: Optional[str] = None,
+               account: str | None = None,
                force: bool = False,
                loop: bool = False) -> None:
 
@@ -378,8 +397,8 @@ def play_sound(sound_event: str,
 
 
 def check_soundfile_path(file_: str,
-                         dirs: Optional[list[Path]] = None
-                         ) -> Optional[Path]:
+                         dirs: list[Path] | None = None
+                         ) -> Path | None:
     '''
     Check if the sound file exists
 
@@ -404,8 +423,8 @@ def check_soundfile_path(file_: str,
     return None
 
 
-def strip_soundfile_path(file_: Union[Path, str],
-                         dirs: Optional[list[Path]] = None,
+def strip_soundfile_path(file_: Path | str,
+                         dirs: list[Path] | None = None,
                          abs_: bool = True):
     '''
     Remove knowns paths from a sound file
@@ -505,7 +524,7 @@ def statuses_unified() -> bool:
     return True
 
 
-def get_full_jid_from_iq(iq_obj: Iq) -> Optional[str]:
+def get_full_jid_from_iq(iq_obj: Iq) -> str | None:
     '''
     Return the full jid (with resource) from an iq
     '''
@@ -515,7 +534,7 @@ def get_full_jid_from_iq(iq_obj: Iq) -> Optional[str]:
     return parse_jid(str(iq_obj.getFrom()))
 
 
-def get_jid_from_iq(iq_obj: Iq) -> Optional[str]:
+def get_jid_from_iq(iq_obj: Iq) -> str | None:
     '''
     Return the jid (without resource) from an iq
     '''
@@ -560,29 +579,45 @@ def get_os_info() -> str:
 
 def message_needs_highlight(text: str, nickname: str, own_jid: str) -> bool:
     '''
-    Check text to see whether any of the words in (muc_highlight_words and
-    nick) appear
+    Check whether 'text' contains 'nickname', 'own_jid', or any string of the
+    'muc_highlight_words' setting.
     '''
-    special_words = app.settings.get('muc_highlight_words').split(';')
-    special_words.append(nickname)
-    special_words.append(own_jid)
-    # Strip empties: ''.split(';') == [''] and would highlight everything.
-    # Also lowercase everything for case insensitive compare.
-    special_words = [word.lower() for word in special_words if word]
+
+    search_strings = app.settings.get('muc_highlight_words').split(';')
+    search_strings.append(nickname)
+    search_strings.append(own_jid)
+
+    search_strings = [word.lower() for word in search_strings if word]
     text = text.lower()
 
-    for special_word in special_words:
-        found_here = text.find(special_word)
-        while found_here > -1:
-            end_here = found_here + len(special_word)
-            if ((found_here == 0 or not text[found_here - 1].isalpha()) and
-                    (end_here == len(text) or not text[end_here].isalpha())):
-                # It is beginning of text or char before is not alpha AND
-                # it is end of text or char after is not alpha
+    for search_string in search_strings:
+        match = text.find(search_string)
+
+        while match > -1:
+            search_end = match + len(search_string)
+
+            if match == 0 and search_end == len(text):
+                # Text contains search_string only (exact match)
                 return True
-            # continue searching
-            start = found_here + 1
-            found_here = text.find(special_word, start)
+
+            char_before_allowed = bool(
+                match == 0 or
+                (not text[match - 1].isalpha() and
+                text[match - 1] not in ('/', '-')))
+
+            if char_before_allowed and search_end == len(text):
+                # search_string found at the end of text and
+                # char before search_string is allowed.
+                return True
+
+            if char_before_allowed and not text[search_end].isalpha():
+                # char_before search_string is allowed and
+                # char_after search_string is not alpha.
+                return True
+
+            start = match + 1
+            match = text.find(search_string, start)
+
     return False
 
 
@@ -639,7 +674,7 @@ def jid_is_blocked(account: str, jid: str) -> bool:
     return jid in client.get_module('Blocking').blocked
 
 
-def get_subscription_request_msg(account: Optional[str] = None) -> str:
+def get_subscription_request_msg(account: str | None = None) -> str:
     if account is None:
         return _('I would like to add you to my contact list.')
 
@@ -652,27 +687,26 @@ def get_subscription_request_msg(account: Optional[str] = None) -> str:
     return Template(message).safe_substitute({'name': app.nicks[account]})
 
 
-def get_retraction_text(account: str,
-                        moderator_jid: str,
-                        reason: Optional[str]) -> str:
+def get_retraction_text(by: str | None,
+                        reason: str | None) -> str:
 
-    client = app.get_client(account)
-    contact = client.get_module('Contacts').get_contact(
-        moderator_jid, groupchat=True)
-    text = _('This message has been retracted by %s.') % contact.name
+    by_text = ''
+    if by is not None:
+        by_text = (' by %s') % by
+    text = _('This message has been moderated%s.') % by_text
     if reason is not None:
         text += ' ' + _('Reason: %s') % reason
     return text
 
 
-def get_global_proxy() -> Optional[ProxyData]:
+def get_global_proxy() -> ProxyData | None:
     proxy_name = app.settings.get('global_proxy')
     if not proxy_name:
         return None
     return get_proxy(proxy_name)
 
 
-def get_account_proxy(account: str, fallback=True) -> Optional[ProxyData]:
+def get_account_proxy(account: str, fallback=True) -> ProxyData | None:
     proxy_name = app.settings.get_account_setting(account, 'proxy')
     if proxy_name:
         return get_proxy(proxy_name)
@@ -682,7 +716,7 @@ def get_account_proxy(account: str, fallback=True) -> Optional[ProxyData]:
     return None
 
 
-def get_proxy(proxy_name: str) -> Optional[ProxyData]:
+def get_proxy(proxy_name: str) -> ProxyData | None:
     try:
         settings = app.settings.get_proxy_settings(proxy_name)
     except ValueError:
@@ -698,7 +732,7 @@ def get_proxy(proxy_name: str) -> Optional[ProxyData]:
                      password=password)
 
 
-def determine_proxy() -> Optional[ProxyData]:
+def determine_proxy() -> ProxyData | None:
     # Use this method to find a proxy for non-account related http requests
     # When there is no global proxy and at least one active account does
     # not use a proxy, we assume no proxy is necessary.
@@ -734,8 +768,8 @@ def call_counter(func):
 
 
 def load_json(path: Path,
-              key: Optional[str] = None,
-              default: Optional[Any] = None) -> Any:
+              key: str | None = None,
+              default: Any | None = None) -> Any:
     try:
         with path.open('r', encoding='utf8') as file:
             json_dict = json.loads(file.read())
@@ -757,7 +791,7 @@ class AdditionalDataDict(collections.UserDict):
             return full_path.split(':')
         return [full_path]
 
-    def set_value(self, full_path: str, key: str, value: Optional[str]) -> None:
+    def set_value(self, full_path: str, key: str, value: str | None) -> None:
         path_childs = self._get_path_childs(full_path)
         _dict = self.data
         for path in path_childs:
@@ -771,7 +805,8 @@ class AdditionalDataDict(collections.UserDict):
     def get_value(self,
                   full_path: str,
                   key: str,
-                  default: Optional[str] = None) -> Optional[Any]:
+                  default: str | bool | None = None
+                  ) -> Any | None:
 
         path_childs = self._get_path_childs(full_path)
         _dict = self.data
@@ -853,9 +888,14 @@ def catch_exceptions(func):
     return func_wrapper
 
 
-def is_registered_uri_scheme(scheme: str) -> bool:
+def is_known_uri_scheme(scheme: str) -> bool:
+    '''
+    `scheme` is lower-case
+    '''
     if scheme in iana.URI_SCHEMES:
         return True
+    if scheme in NONREGISTERED_URI_SCHEMES:
+        return True
     return scheme in app.settings.get('additional_uri_schemes').split()
 
 
@@ -896,8 +936,8 @@ def parse_uri(uri: str) -> URI:
 
     scheme = urlparts.scheme  # urlparse is expected to return it in lower case
 
-    if not is_registered_uri_scheme(scheme):
-        return URI(URIType.INVALID, uri, data={'error': 'Unregistered scheme'})
+    if not is_known_uri_scheme(scheme):
+        return URI(URIType.INVALID, uri, data={'error': 'Unknown scheme'})
 
     if scheme in ('https', 'http'):
         if not urlparts.netloc:
@@ -998,7 +1038,7 @@ _xmpp_query_type_handlers = {
 
 
 @catch_exceptions
-def open_uri(uri: Union[URI, str], account: Optional[str] = None) -> None:
+def open_uri(uri: URI | str, account: str | None = None) -> None:
     if not isinstance(uri, URI):
         uri = parse_uri(uri)
 
@@ -1067,7 +1107,7 @@ def open_file_uri(uri: str) -> None:
         if sys.platform != 'win32':
             Gio.AppInfo.launch_default_for_uri(uri)
         else:
-            os.startfile(uri)
+            os.startfile(uri)  # noqa: S606
     except Exception as err:
         log.info("Couldn't open file URI %s: %s", uri, err)
 
@@ -1091,7 +1131,7 @@ def show_in_folder(path: Path) -> None:
         open_directory(path.parent)
 
 
-def filesystem_path_from_uri(uri: str) -> Optional[Path]:
+def filesystem_path_from_uri(uri: str) -> Path | None:
     puri = parse_uri(uri)
     if puri.type != URIType.FILE:
         return None
@@ -1102,7 +1142,7 @@ def filesystem_path_from_uri(uri: str) -> Optional[Path]:
     return Path(puri.data['path'])
 
 
-def get_file_path_from_dnd_dropped_uri(text: str) -> Optional[Path]:
+def get_file_path_from_dnd_dropped_uri(text: str) -> Path | None:
     uri = text.strip('\r\n\x00')  # remove \r\n and NULL
     return filesystem_path_from_uri(uri)
 
@@ -1142,7 +1182,7 @@ def geo_provider_from_location(lat: str, lon: str) -> str:
     return f'https://www.openstreetmap.org/?mlat={lat}&mlon={lon}&zoom=16'
 
 
-def get_resource(account: str) -> Optional[str]:
+def get_resource(account: str) -> str | None:
     resource = app.settings.get_account_setting(account, 'resource')
     if not resource:
         return None
@@ -1154,7 +1194,7 @@ def get_resource(account: str) -> Optional[str]:
     return resource
 
 
-def get_default_muc_config() -> dict[str, Union[bool, str]]:
+def get_default_muc_config() -> dict[str, bool | str]:
     return {
         # XEP-0045 options
         'muc#roomconfig_allowinvites': True,
@@ -1168,6 +1208,7 @@ def get_default_muc_config() -> dict[str, Union[bool, str]]:
         # Ejabberd options
         'allow_voice_requests': False,
         'public_list': False,
+        'mam': True,
 
         # Prosody options
         '{http://prosody.im/protocol/muc}roomconfig_allowmemberinvites': True,
@@ -1175,7 +1216,7 @@ def get_default_muc_config() -> dict[str, Union[bool, str]]:
     }
 
 
-def validate_jid(jid: Union[str, JID], type_: Optional[str] = None) -> JID:
+def validate_jid(jid: str | JID, type_: str | None = None) -> JID:
     try:
         jid = JID.from_string(str(jid))
     except InvalidJid as error:
@@ -1193,7 +1234,7 @@ def validate_jid(jid: Union[str, JID], type_: Optional[str] = None) -> JID:
     raise ValueError(f'Not a {type_} JID')
 
 
-def to_user_string(error: Union[CommonError, StanzaError]) -> str:
+def to_user_string(error: CommonError | StanzaError) -> str:
     text = error.get_text(get_rfc5646_lang())
     if text:
         return text
@@ -1219,7 +1260,7 @@ def get_groupchat_name(client: types.Client, jid: JID) -> str:
 
 def is_affiliation_change_allowed(self_contact: types.GroupchatParticipant,
                                   contact: types.GroupchatParticipant,
-                                  target_aff: Union[str, Affiliation]) -> bool:
+                                  target_aff: str | Affiliation) -> bool:
     if isinstance(target_aff, str):
         target_aff = Affiliation(target_aff)
 
@@ -1262,13 +1303,13 @@ def get_tls_error_phrases(tls_errors: set[Gio.TlsCertificateFlags]
 
 
 class Observable:
-    def __init__(self, log_: Union[logging.Logger, LogAdapter, None] = None):
+    def __init__(self, log_: logging.Logger | LogAdapter | None = None):
         self._log = log_
         self._callbacks: types.ObservableCbDict = defaultdict(list)
 
     def __disconnect(self,
                      obj: Any,
-                     signals: Optional[set[str]] = None
+                     signals: set[str] | None = None
                      ) -> None:
 
         def _remove(handlers: list[weakref.WeakMethod[types.AnyCallableT]]
@@ -1292,7 +1333,7 @@ class Observable:
 
     def multi_disconnect(self,
                          obj: Any,
-                         signals: Optional[set[str]]
+                         signals: set[str] | None
                          ) -> None:
 
         self.__disconnect(obj, signals)
@@ -1344,8 +1385,8 @@ class Observable:
 def write_file_async(
         path: Path,
         data: bytes,
-        callback: Callable[[bool, Optional[GLib.Error], Any], Any],
-        user_data: Optional[Any] = None):
+        callback: Callable[[bool, GLib.Error | None, Any], Any],
+        user_data: Any | None = None):
 
     def _on_write_finished(outputstream: Gio.OutputStream,
                            result: Gio.AsyncResult,
@@ -1381,10 +1422,10 @@ def write_file_async(
 
 
 def load_file_async(path: Path,
-                    callback: Callable[[Optional[bytes],
-                                        Optional[GLib.Error],
-                                        Optional[Any]], Any],
-                    user_data: Optional[Any] = None) -> None:
+                    callback: Callable[[bytes | None,
+                                        GLib.Error | None,
+                                        Any | None], Any],
+                    user_data: Any | None = None) -> None:
 
     def _on_load_finished(file: Gio.File,
                           result: Gio.AsyncResult) -> None:
@@ -1406,9 +1447,10 @@ def get_x509_cert_from_gio_cert(cert: Gio.TlsCertificate) -> x509.Certificate:
         glib_bytes.get_data(), default_backend())
 
 
-def get_custom_host(account: str) -> Optional[tuple[str,
-                                                    ConnectionProtocol,
-                                                    ConnectionType]]:
+def get_custom_host(
+    account: str
+) -> tuple[str, ConnectionProtocol, ConnectionType] | None:
+
     if not app.settings.get_account_setting(account, 'use_custom_host'):
         return None
     host = app.settings.get_account_setting(account, 'custom_host')
@@ -1445,7 +1487,7 @@ def get_idle_status_message(state: str, status_message: str) -> str:
     }
 
 
-def get_group_chat_nick(account: str, room_jid: Union[JID, str]) -> str:
+def get_group_chat_nick(account: str, room_jid: JID | str) -> str:
     client = app.get_client(account)
 
     bookmark = client.get_module('Bookmarks').get_bookmark(room_jid)
@@ -1490,7 +1532,7 @@ def get_os_name() -> str:
 
 def get_os_version() -> str:
     if sys.platform in ('win32', 'darwin'):
-        return platform.release()
+        return platform.version()
     if os.name == 'posix':
         try:
             import distro
@@ -1510,6 +1552,23 @@ def get_glib_version() -> str:
                               GLib.MICRO_VERSION]))
 
 
+def get_soup_version() -> str:
+    return '.'.join(map(str, [Soup.get_major_version(),
+                              Soup.get_minor_version(),
+                              Soup.get_micro_version()]))
+
+
+def package_version(requirement: str) -> bool:
+    req = Requirement(requirement)
+
+    try:
+        installed_version = importlib.metadata.version(req.name)
+    except importlib.metadata.PackageNotFoundError:
+        return False
+
+    return installed_version in req.specifier
+
+
 def make_path_from_jid(base_path: Path, jid: JID) -> Path:
     assert jid.domain is not None
     domain = jid.domain[:50]
