@@ -14,7 +14,6 @@
 
 from typing import Literal
 from typing import TypedDict
-from typing import Union
 
 import uuid
 
@@ -161,7 +160,7 @@ StringSettings = Literal[
 ]
 
 AllSettings = Literal[BoolSettings, IntSettings, StringSettings]
-AllSettingsT = Union[str, int, bool, list[str]]
+AllSettingsT = str | int | bool | list[str]
 
 APP_SETTINGS = {
     'additional_uri_schemes': '',
@@ -208,7 +207,7 @@ APP_SETTINGS = {
     'gc_sync_threshold_private_default': 0,
     'gc_sync_threshold_public_default': 1,
     'global_proxy': '',
-    'groupchat_roster_width': 210,
+    'groupchat_roster_width': 250,
     'hide_groupchat_occupants_list': False,
     'ignore_incoming_attention': False,
     'is_window_visible': True,
@@ -275,7 +274,7 @@ APP_SETTINGS = {
     'use_keyring': True,
     'use_kib_mib': False,
     'use_libappindicator': True,
-    'use_speller': False,
+    'use_speller': True,
     'use_stun_server': False,
     'use_urgency_hint': True,
     'video_framerate': '',
@@ -314,7 +313,8 @@ BoolAccountSettings = Literal[
     'test_ft_proxies_on_startup',
     'use_custom_host',
     'use_ft_proxies',
-    'use_plain_connection'
+    'use_plain_connection',
+    'omemo_blind_trust'
 ]
 
 
@@ -389,7 +389,7 @@ AllGroupChatSettings = Literal[BoolGroupChatSettings,
                                IntGroupChatSettings,
                                StringGroupChatSettings]
 
-AllGroupChatSettingsT = Union[str, int, bool]
+AllGroupChatSettingsT = str | int | bool
 
 
 BoolContactSettings = Literal[
@@ -406,7 +406,7 @@ StringContactSettings = Literal[
 AllContactSettings = Literal[BoolContactSettings,
                              StringContactSettings]
 
-AllContactSettingsT = Union[str, bool]
+AllContactSettingsT = str | bool
 
 
 ACCOUNT_SETTINGS = {
@@ -436,6 +436,7 @@ ACCOUNT_SETTINGS = {
         'default_workspace': '',
         'enable_gssapi': False,
         'enable_security_labels': False,
+        'encryption_default': '',
         'file_transfer_proxies': '',
         'filetransfer_preference': 'httpupload',
         'ft_send_local_ips': True,
@@ -476,10 +477,11 @@ ACCOUNT_SETTINGS = {
         'zeroconf_first_name': '',
         'zeroconf_jabber_id': '',
         'zeroconf_last_name': '',
+        'omemo_blind_trust': True,
     },
 
     'contact': {
-        'encryption': '',
+        'encryption': HAS_ACCOUNT_DEFAULT,
         'mute_until': '',
         'send_chatstate': HAS_ACCOUNT_DEFAULT,
         'send_marker': HAS_ACCOUNT_DEFAULT,
@@ -517,7 +519,7 @@ class OpenChatSettingDetails(TypedDict):
 
 
 OpenChatsSettingT = list[OpenChatSettingDetails]
-AllWorkspaceSettingsT = Union[str, OpenChatsSettingT]
+AllWorkspaceSettingsT = str | OpenChatsSettingT
 
 
 class WorkspaceSettings(TypedDict):
@@ -669,7 +671,6 @@ ADVANCED_SETTINGS = {
             'account in the Accounts window. BE CAREFUL, when you are logged '
             'in with a negative priority, you will NOT receive any message '
             'from your server.'),
-        'escape_key_closes': _('Close a chat by pressing the Escape key'),
         'file_transfers_port': '',
         'ft_add_hosts_to_send': _(
             'List of send hosts (comma separated) in '
