@@ -17,9 +17,7 @@ from __future__ import annotations
 from typing import Any
 from typing import cast
 from typing import Literal
-from typing import Optional
 from typing import overload
-from typing import Union
 
 import logging
 
@@ -38,6 +36,8 @@ from nbxmpp.errors import RegisterStanzaError
 from nbxmpp.errors import StanzaError
 from nbxmpp.protocol import JID
 from nbxmpp.protocol import validate_domainpart
+from nbxmpp.stringprep import saslprep
+from nbxmpp.structs import ProxyData
 from nbxmpp.task import Task
 
 from gajim.common import app
@@ -47,24 +47,28 @@ from gajim.common.const import GIO_TLS_ERRORS
 from gajim.common.const import SASL_ERRORS
 from gajim.common.events import StanzaReceived
 from gajim.common.events import StanzaSent
+from gajim.common.helpers import get_global_proxy
 from gajim.common.helpers import get_proxy
 from gajim.common.helpers import open_uri
 from gajim.common.helpers import validate_jid
 from gajim.common.i18n import _
+from gajim.common.util.http import create_http_session
 
-from .accounts import AccountsWindow
-from .assistant import Assistant
-from .assistant import ErrorPage
-from .assistant import Page
-from .assistant import ProgressPage
-from .assistant import SuccessPage
-from .builder import get_builder
-from .dataform import DataFormWidget
-from .util import get_app_window
-from .util import get_color_for_account
-from .util import open_window
+from gajim.gtk.accounts import AccountsWindow
+from gajim.gtk.assistant import Assistant
+from gajim.gtk.assistant import ErrorPage
+from gajim.gtk.assistant import Page
+from gajim.gtk.assistant import ProgressPage
+from gajim.gtk.assistant import SuccessPage
+from gajim.gtk.builder import get_builder
+from gajim.gtk.dataform import DataFormWidget
+from gajim.gtk.util import get_app_window
+from gajim.gtk.util import get_color_for_account
+from gajim.gtk.util import open_window
 
-log = logging.getLogger('gajim.gui.account_wizard')
+CustomHostT = tuple[str, ConnectionProtocol, ConnectionType]
+
+log = logging.getLogger('gajim.gtk.account_wizard')
 
 
 class AccountWizard(Assistant):
@@ -102,7 +106,7 @@ class AccountWizard(Assistant):
 
         self.update_proxy_list()
 
-        self._client: Optional[Client] = None
+        self._client: Client | None = None
         self._method: str = 'login'
 
     @overload
@@ -258,9 +262,18 @@ class AccountWizard(Assistant):
     def update_proxy_list(self) -> None:
         self.get_page('advanced').update_proxy_list()
 
+    def _get_proxy_data(self, advanced: bool) -> ProxyData | None:
+        if advanced:
+            proxy_name = self.get_page('advanced').get_proxy()
+            proxy_data = get_proxy(proxy_name)
+            if proxy_data is not None:
+                return proxy_data
+
+        return get_global_proxy()
+
     def _get_base_client(self,
                          domain: str,
-                         username: Optional[str],
+                         username: str | None,
                          mode: Mode,
                          advanced: bool,
                          ignore_all_errors: bool
@@ -279,11 +292,9 @@ class AccountWizard(Assistant):
             if custom_host is not None:
                 client.set_custom_host(*custom_host)
 
-            proxy_name = self.get_page('advanced').get_proxy()
-            proxy_data = get_proxy(proxy_name)
-            if proxy_data is not None:
-                client.set_proxy(proxy_data)
-
+        proxy_data = self._get_proxy_data(advanced)
+        client.set_proxy(proxy_data)
+        client.set_http_session(create_http_session(proxy=proxy_data))
         client.subscribe('disconnected', self._on_disconnected)
         client.subscribe('connection-failed', self._on_connection_failed)
         client.subscribe('stanza-sent', self._on_stanza_sent)
@@ -556,11 +567,11 @@ class AccountWizard(Assistant):
         self.get_page('form').remove_form()
         self._disconnect()
 
-    def _set_error_text(self,
-                        error: Union[StanzaError,
-                                     RegisterStanzaError,
-                                     MalformedStanzaError]
-                        ) -> None:
+    def _set_error_text(
+        self,
+        error: StanzaError | RegisterStanzaError | MalformedStanzaError
+    ) -> None:
+
         error_text = error.get_text()
         if not error_text:
             error_text = _('The server rejected the registration '
@@ -664,12 +675,44 @@ class Login(Page):
                 Gtk.EntryIconPosition.SECONDARY, _('Invalid Address'))
             return False
 
+        for account in app.settings.get_accounts():
+            name = app.settings.get_account_setting(account, 'name')
+            hostname = app.settings.get_account_setting(account, 'hostname')
+            if address == f'{name}@{hostname}':
+                self._show_icon(True)
+                self._ui.log_in_address_entry.set_icon_tooltip_text(
+                    Gtk.EntryIconPosition.SECONDARY,
+                    _('This account has already been added'))
+                return False
+
         self._show_icon(False)
         return True
 
+    def _hide_password_warning(self) -> None:
+        self._ui.log_in_password_entry.set_icon_from_icon_name(
+            Gtk.EntryIconPosition.SECONDARY, None)
+        self._ui.log_in_password_entry.set_icon_tooltip_text(
+            Gtk.EntryIconPosition.SECONDARY, None)
+
+    def _show_password_warning(self) -> None:
+        self._ui.log_in_password_entry.set_icon_from_icon_name(
+            Gtk.EntryIconPosition.SECONDARY, 'dialog-warning-symbolic')
+        self._ui.log_in_password_entry.set_icon_tooltip_text(
+            Gtk.EntryIconPosition.SECONDARY,
+            _('Password contains prohibited characters'))
+
+    def _validate_password(self, password: str) -> str | None:
+        self._hide_password_warning()
+        try:
+            return saslprep(password)
+        except Exception:
+            self._show_password_warning()
+            return None
+
     def _set_complete(self, *args: Any) -> None:
         address = self._validate_jid(self._ui.log_in_address_entry.get_text())
         password = self._ui.log_in_password_entry.get_text()
+        password = self._validate_password(password)
         self._ui.log_in_button.set_sensitive(bool(address and password))
 
     def is_advanced(self) -> bool:
@@ -677,7 +720,7 @@ class Login(Page):
 
     def get_credentials(self) -> tuple[str, str]:
         data = (self._ui.log_in_address_entry.get_text(),
-                self._ui.log_in_password_entry.get_text())
+                saslprep(self._ui.log_in_password_entry.get_text()))
         return data
 
 
@@ -797,9 +840,7 @@ class AdvancedSettings(Page):
         active = self._ui.proxies_combobox.get_active()
         return self._ui.proxies_combobox.get_model()[active][0]
 
-    def get_custom_host(self) -> Optional[tuple[str,
-                                                ConnectionProtocol,
-                                                ConnectionType]]:
+    def get_custom_host(self) -> CustomHostT | None:
         host = self._ui.custom_host_entry.get_text()
         port = self._ui.custom_port_entry.get_text()
         if not host or not port:
@@ -898,8 +939,8 @@ class SecurityWarning(Page):
     def __init__(self) -> None:
         Page.__init__(self)
         self.title: str = _('Security Warning')
-        self._cert: Optional[Gio.TlsCertificate] = None
-        self._domain: Optional[str] = None
+        self._cert: Gio.TlsCertificate | None = None
+        self._domain: str | None = None
 
         self._ui = get_builder('account_wizard.ui')
         self.pack_start(self._ui.security_warning_box, True, True, 0)
@@ -907,7 +948,7 @@ class SecurityWarning(Page):
         self.show_all()
 
     @property
-    def cert(self) -> Optional[Gio.TlsCertificate]:
+    def cert(self) -> Gio.TlsCertificate | None:
         return self._cert
 
     def set_warning(self,
@@ -964,7 +1005,7 @@ class Form(Page):
         self.set_valign(Gtk.Align.FILL)
         self.complete: bool = False
         self.title: str = _('Create Account')
-        self._current_form: Optional[Any] = None
+        self._current_form: Any | None = None
 
         heading = Gtk.Label(label=_('Create Account'))
         heading.get_style_context().add_class('large-header')
@@ -980,7 +1021,19 @@ class Form(Page):
     def has_form(self) -> bool:
         return self._current_form is not None
 
+    def _validate_password(self, password: str) -> str | None:
+        try:
+            return saslprep(password)
+        except Exception:
+            self._show_password_warning()
+            return None
+
     def _on_is_valid(self, _widget: DataFormWidget, is_valid: bool) -> None:
+        try:
+            self.get_credentials()
+        except Exception:
+            is_valid = False
+
         self.complete = is_valid
         self.update_page_complete()
 
@@ -998,7 +1051,7 @@ class Form(Page):
     def get_credentials(self) -> tuple[str, str]:
         assert self._current_form is not None
         return (self._current_form.get_form()['username'].value,
-                self._current_form.get_form()['password'].value)
+                saslprep(self._current_form.get_form()['password'].value))
 
     def get_submit_form(self) -> Any:
         assert self._current_form is not None
@@ -1027,7 +1080,7 @@ class Redirect(Page):
     def __init__(self) -> None:
         Page.__init__(self)
         self.title: str = _('Redirect')
-        self._link: Optional[str] = None
+        self._link: str | None = None
 
         self._ui = get_builder('account_wizard.ui')
         self.pack_start(self._ui.redirect_box, True, True, 0)
@@ -1054,10 +1107,10 @@ class Success(SuccessPage):
         self.set_title(_('Account Added'))
         self.set_heading(_('Account has been added successfully'))
 
-        self._account: Optional[str] = None
-        self._our_jid: Optional[str] = None
-        self._label: Optional[str] = None
-        self._color: Optional[str] = None
+        self._account: str | None = None
+        self._our_jid: str | None = None
+        self._label: str | None = None
+        self._color: str | None = None
 
         self._ui = get_builder('account_wizard.ui')
         self.pack_start(self._ui.account_label_box, True, True, 0)
@@ -1081,7 +1134,7 @@ class Success(SuccessPage):
         self._save_config()
 
     @property
-    def account(self) -> Optional[str]:
+    def account(self) -> str | None:
         return self._account
 
     def _add_css_provider(self) -> Gtk.CssProvider:
