@@ -36,12 +36,14 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
 from typing import cast
-from typing import Optional
 
 import os
 import sys
+from collections.abc import Callable
+from datetime import datetime
+from datetime import timedelta
+from datetime import timezone
 from urllib.parse import unquote
 
 from gi.repository import Gdk
@@ -62,32 +64,36 @@ from gajim.common import ged
 from gajim.common import idle
 from gajim.common.application import CoreApplication
 from gajim.common.const import GAJIM_FAQ_URI
+from gajim.common.const import GAJIM_PRIVACY_POLICY_URI
 from gajim.common.const import GAJIM_SUPPORT_JID
 from gajim.common.const import GAJIM_WIKI_URI
 from gajim.common.exceptions import GajimGeneralException
 from gajim.common.helpers import load_json
 from gajim.common.helpers import open_uri
 from gajim.common.i18n import _
-from gajim.gui import menus
-from gajim.gui import structs
-from gajim.gui.about import AboutDialog
-from gajim.gui.accounts import AccountsWindow
-from gajim.gui.avatar import AvatarStorage
-from gajim.gui.builder import get_builder
-from gajim.gui.const import ACCOUNT_ACTIONS
-from gajim.gui.const import ALWAYS_ACCOUNT_ACTIONS
-from gajim.gui.const import APP_ACTIONS
-from gajim.gui.const import FEATURE_ACCOUNT_ACTIONS
-from gajim.gui.const import ONLINE_ACCOUNT_ACTIONS
-from gajim.gui.dialogs import ConfirmationDialog
-from gajim.gui.dialogs import DialogButton
-from gajim.gui.dialogs import ShortcutsWindow
-from gajim.gui.discovery import ServiceDiscoveryWindow
-from gajim.gui.start_chat import StartChatDialog
-from gajim.gui.util import get_app_window
-from gajim.gui.util import get_app_windows
-from gajim.gui.util import load_user_iconsets
-from gajim.gui.util import open_window
+from gajim.common.modules.contacts import ResourceContact
+
+from gajim.gtk import menus
+from gajim.gtk import structs
+from gajim.gtk.about import AboutDialog
+from gajim.gtk.accounts import AccountsWindow
+from gajim.gtk.avatar import AvatarStorage
+from gajim.gtk.builder import get_builder
+from gajim.gtk.const import ACCOUNT_ACTIONS
+from gajim.gtk.const import ALWAYS_ACCOUNT_ACTIONS
+from gajim.gtk.const import APP_ACTIONS
+from gajim.gtk.const import FEATURE_ACCOUNT_ACTIONS
+from gajim.gtk.const import MuteState
+from gajim.gtk.const import ONLINE_ACCOUNT_ACTIONS
+from gajim.gtk.dialogs import ConfirmationDialog
+from gajim.gtk.dialogs import DialogButton
+from gajim.gtk.dialogs import ShortcutsWindow
+from gajim.gtk.discovery import ServiceDiscoveryWindow
+from gajim.gtk.start_chat import StartChatDialog
+from gajim.gtk.util import get_app_window
+from gajim.gtk.util import get_app_windows
+from gajim.gtk.util import load_user_iconsets
+from gajim.gtk.util import open_window
 
 ActionListT = list[tuple[str,
                          Callable[[Gio.SimpleAction, GLib.Variant], Any]]]
@@ -188,14 +194,9 @@ class GajimApplication(Gtk.Application, CoreApplication):
             GLib.OptionArg.NONE,
             _('Start a new chat'))
 
-        self.add_main_option(
-            'show', 0,
-            GLib.OptionFlags.NONE,
-            GLib.OptionArg.NONE,
-            _('Show Gajim'))
-
         self.add_main_option_entries(self._get_remaining_entry())
 
+        self.connect('activate', self._on_activate)
         self.connect('handle-local-options', self._handle_local_options)
         self.connect('command-line', self._command_line)
         self.connect('shutdown', self._shutdown)
@@ -232,7 +233,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
         builder = get_builder('application_menu.ui')
         self.set_menubar(cast(Gio.Menu, builder.get_object('menubar')))
 
-        from gajim.gui import notification
+        from gajim.gtk import notification
         notification.init()
 
         self.avatar_storage = AvatarStorage()
@@ -247,7 +248,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
         self.interface = Interface()
         self.interface.run(self)
 
-        from gajim.gui.status_icon import StatusIcon
+        from gajim.gtk.status_icon import StatusIcon
         self.systray = StatusIcon()
 
         self._add_app_actions()
@@ -263,7 +264,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
                             ged.CORE,
                             self._on_feature_discovered)
 
-        from gajim.gui.main import MainWindow
+        from gajim.gtk.main import MainWindow
         MainWindow()
 
         GLib.timeout_add(100, self._auto_connect)
@@ -338,21 +339,25 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
         remote_commands = [
             ('start-chat', GLib.Variant('as', ['', ''])),
-            ('show', None)
         ]
 
-        remaining = options.lookup_value(GLib.OPTION_REMAINING,
-                                         GLib.VariantType.new('as'))
-
         for cmd, parameter in remote_commands:
             if options.contains(cmd):
                 self.activate_action(cmd, parameter)
                 return 0
 
+        remaining = options.lookup_value(GLib.OPTION_REMAINING,
+                                         GLib.VariantType.new('as'))
+
         if remaining is not None:
             self._open_uris(remaining.unpack())
             return 0
 
+        if not options.contains('is-first-startup'):
+            # If no commands have been handled and it's not the first
+            # startup, raise the application.
+            self.activate()
+
         return 0
 
     def _handle_local_options(self,
@@ -368,7 +373,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
             # Incorporate profile name into application id
             # to have a single app instance for each profile.
             profile = profile.get_string()
-            app_id = '%s.%s' % (self.get_application_id(), profile)
+            app_id = f'{self.get_application_id()}.{profile}'
             self.set_application_id(app_id)
             configpaths.set_profile(profile)
 
@@ -378,10 +383,14 @@ class GajimApplication(Gtk.Application, CoreApplication):
                   'The primary instance will handle remote commands')
             return -1
 
+        options.insert_value('is-first-startup',  GLib.Variant('b', True))
         self._core_command_line(options)
         self._startup()
         return -1
 
+    def _on_activate(self, _application: Gtk.Application) -> None:
+        app.window.show()
+
     def _add_app_actions(self) -> None:
         for action in APP_ACTIONS:
             action_name, variant = action
@@ -408,6 +417,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
             ('join-support-chat', self._on_join_support_chat),
             ('about', self._on_about_action),
             ('faq', self._on_faq_action),
+            ('privacy-policy', self._on_privacy_policy_action),
             ('start-chat', self._on_new_chat_action),
             ('accounts', self._on_accounts_action),
             ('add-contact', self._on_add_contact_action),
@@ -417,7 +427,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
             ('create-groupchat', self._on_create_groupchat_action),
             ('forget-groupchat', self._on_forget_groupchat_action),
             ('open-chat', self._on_open_chat_action),
-            ('show', self._on_show),
+            ('mute-chat', self._on_mute_chat_action),
         ]
 
         for action in actions:
@@ -470,7 +480,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     def set_action_state(self, action_name: str, state: bool) -> None:
         action = self.lookup_action(action_name)
-        assert action is not None
+        assert isinstance(action, Gio.SimpleAction)
         action.set_enabled(state)
 
     def set_account_actions_state(self,
@@ -522,10 +532,10 @@ class GajimApplication(Gtk.Application, CoreApplication):
                        username: str,
                        domain: str,
                        password: str,
-                       proxy_name: str,
+                       proxy_name: str | None,
                        custom_host: tuple[str,
                                           ConnectionProtocol,
-                                          ConnectionType],
+                                          ConnectionType] | None,
                        anonymous: bool = False
                        ) -> None:
 
@@ -543,7 +553,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
         # Action must be added before account window is updated
         self.add_account_actions(account)
 
-        window = cast(AccountsWindow, get_app_window('AccountsWindow'))
+        window = cast(AccountsWindow | None, get_app_window('AccountsWindow'))
         if window is not None:
             window.add_account(account)
 
@@ -551,7 +561,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
         CoreApplication.enable_account(self, account)
         menus.build_accounts_menu()
         self.update_app_actions_state()
-        window = cast(AccountsWindow, get_app_window('AccountsWindow'))
+        window = cast(AccountsWindow | None, get_app_window('AccountsWindow'))
         if window is not None:
             window.enable_account(account, True)
 
@@ -573,7 +583,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
         self.remove_account_actions(account)
 
-        window = cast(AccountsWindow, get_app_window('AccountsWindow'))
+        window = cast(AccountsWindow | None, get_app_window('AccountsWindow'))
         if window is not None:
             window.remove_account(account)
 
@@ -589,12 +599,12 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     @staticmethod
     def _on_preferences_action(_action: Gio.SimpleAction,
-                               _param: Optional[GLib.Variant]) -> None:
+                               _param: GLib.Variant | None) -> None:
         open_window('Preferences')
 
     @staticmethod
     def _on_plugins_action(_action: Gio.SimpleAction,
-                           _param: Optional[GLib.Variant]) -> None:
+                           _param: GLib.Variant | None) -> None:
         open_window('PluginsWindow')
 
     @staticmethod
@@ -613,7 +623,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     @staticmethod
     def _on_quit_action(_action: Gio.SimpleAction,
-                        _param: Optional[GLib.Variant]) -> None:
+                        _param: GLib.Variant | None) -> None:
         app.window.quit()
 
     @staticmethod
@@ -659,13 +669,13 @@ class GajimApplication(Gtk.Application, CoreApplication):
     def _on_add_contact_account_action(_action: Gio.SimpleAction,
                                        param: GLib.Variant) -> None:
         account, jid = param.get_strv()
-        if jid is not None:
+        if jid:
             jid = JID.from_string(jid)
         open_window('AddContact', account=account or None, jid=jid or None)
 
     @staticmethod
     def _on_add_account_action(_action: Gio.SimpleAction,
-                               _param: Optional[GLib.Variant]) -> None:
+                               _param: GLib.Variant | None) -> None:
         open_window('AccountWizard')
 
     @staticmethod
@@ -711,22 +721,22 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     @staticmethod
     def _on_xml_console_action(_action: Gio.SimpleAction,
-                               _param: Optional[GLib.Variant]) -> None:
+                               _param: GLib.Variant | None) -> None:
         open_window('XMLConsoleWindow')
 
     @staticmethod
     def _on_manage_proxies_action(_action: Gio.SimpleAction,
-                                  _param: Optional[GLib.Variant]) -> None:
+                                  _param: GLib.Variant | None) -> None:
         open_window('ManageProxies')
 
     @staticmethod
     def _on_content_action(_action: Gio.SimpleAction,
-                           _param: Optional[GLib.Variant]) -> None:
+                           _param: GLib.Variant | None) -> None:
         open_uri(GAJIM_WIKI_URI)
 
     @staticmethod
     def _on_join_support_chat(_action: Gio.SimpleAction,
-                              _param: Optional[GLib.Variant]) -> None:
+                              _param: GLib.Variant | None) -> None:
         accounts = app.settings.get_active_accounts()
         if len(accounts) == 1:
             app.window.show_add_join_groupchat(
@@ -736,27 +746,32 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     @staticmethod
     def _on_faq_action(_action: Gio.SimpleAction,
-                       _param: Optional[GLib.Variant]) -> None:
+                       _param: GLib.Variant | None) -> None:
         open_uri(GAJIM_FAQ_URI)
 
+    @staticmethod
+    def _on_privacy_policy_action(_action: Gio.SimpleAction,
+                                  _param: GLib.Variant | None) -> None:
+        open_uri(GAJIM_PRIVACY_POLICY_URI)
+
     @staticmethod
     def _on_shortcuts_action(_action: Gio.SimpleAction,
-                             _param: Optional[GLib.Variant]) -> None:
+                             _param: GLib.Variant | None) -> None:
         ShortcutsWindow()
 
     @staticmethod
     def _on_features_action(_action: Gio.SimpleAction,
-                            _param: Optional[GLib.Variant]) -> None:
+                            _param: GLib.Variant | None) -> None:
         open_window('Features')
 
     @staticmethod
     def _on_about_action(_action: Gio.SimpleAction,
-                         _param: Optional[GLib.Variant]) -> None:
+                         _param: GLib.Variant | None) -> None:
         AboutDialog()
 
     @staticmethod
     def _on_file_transfer_action(_action: Gio.SimpleAction,
-                                 _param: Optional[GLib.Variant]) -> None:
+                                 _param: GLib.Variant | None) -> None:
 
         ft = app.interface.instances['file_transfers']
         if ft.window.get_property('visible'):
@@ -813,6 +828,23 @@ class GajimApplication(Gtk.Application, CoreApplication):
         account, jid = param.get_strv()
         app.window.start_chat_from_jid(account, jid)
 
+    @staticmethod
+    @structs.actionfunction
+    def _on_mute_chat_action(_action: Gio.SimpleAction,
+                             params: structs.MuteContactParam
+                             ) -> None:
+
+        client = app.get_client(params.account)
+        contact = client.get_module('Contacts').get_contact(params.jid)
+        assert not isinstance(contact, ResourceContact)
+
+        if params.state == MuteState.UNMUTE:
+            contact.settings.set('mute_until', None)
+            return
+
+        until = datetime.now(timezone.utc) + timedelta(minutes=params.state)
+        contact.settings.set('mute_until', until.isoformat())
+
     @staticmethod
     @structs.actionfunction
     def _on_remove_history_action(_action: Gio.SimpleAction,
@@ -820,10 +852,11 @@ class GajimApplication(Gtk.Application, CoreApplication):
                                   ) -> None:
         def _remove() -> None:
             app.storage.archive.remove_history(params.account, params.jid)
+
+            app.window.clear_chat_list_row(params.account, params.jid)
             control = app.window.get_control()
-            if params.jid is not None:
-                if not control.is_loaded(params.account, params.jid):
-                    return
+            if not control.is_loaded(params.account, params.jid):
+                return
 
             control.reset_view()
 
@@ -850,7 +883,3 @@ class GajimApplication(Gtk.Application, CoreApplication):
         client.get_module('Bookmarks').remove(params.jid)
 
         app.storage.archive.remove_history(params.account, params.jid)
-
-    @staticmethod
-    def _on_show(_action: Gio.SimpleAction, param: GLib.Variant) -> None:
-        app.window.show()
