@@ -15,7 +15,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import pickle
 from datetime import datetime
@@ -63,6 +62,10 @@ class ChatListRow(Gtk.ListBoxRow):
             GObject.SignalFlags.RUN_LAST,
             None,
             ()),
+        'context-menu-state-changed': (
+            GObject.SignalFlags.RUN_LAST,
+            None,
+            (bool,)),
     }
 
     def __init__(self,
@@ -88,12 +91,15 @@ class ChatListRow(Gtk.ListBoxRow):
         self._client = app.get_client(account)
 
         self.contact = self._client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            self.contact,
+            BareContact | GroupchatContact | GroupchatParticipant)
         self._connect_contact_signals()
 
         self.contact_name: str = self.contact.name
         self.timestamp: float = 0
-        self.stanza_id: Optional[str] = None
-        self.message_id: Optional[str] = None
+        self.stanza_id: str | None = None
+        self.message_id: str | None = None
 
         self._unread_count: int = 0
         self._needs_muc_highlight: bool = False
@@ -161,7 +167,7 @@ class ChatListRow(Gtk.ListBoxRow):
                     reason = line.additional_data.get_value(
                         'retracted', 'reason')
                     message_text = get_retraction_text(
-                        self.account, retracted_by, reason)
+                        retracted_by, reason)
 
             me_nickname = None
             if line.kind in (KindConstant.CHAT_MSG_SENT,
@@ -209,7 +215,8 @@ class ChatListRow(Gtk.ListBoxRow):
 
     @property
     def unread_count(self) -> int:
-        if (self.contact.is_groupchat and not self.contact.can_notify() and
+        if (isinstance(self.contact, GroupchatContact) and
+                not self.contact.can_notify() and
                 not self._needs_muc_highlight):
             return 0
         return self._unread_count
@@ -220,14 +227,14 @@ class ChatListRow(Gtk.ListBoxRow):
         self._update_unread()
         self.emit('unread-changed')
 
-    def get_header_type(self) -> Optional[RowHeaderType]:
+    def get_header_type(self) -> RowHeaderType | None:
         header = self.get_header()
         if header is None:
             return None
         assert isinstance(header, RowHeader)
         return header.type
 
-    def set_header_type(self, header_type: Optional[RowHeaderType]) -> None:
+    def set_header_type(self, header_type: RowHeaderType | None) -> None:
         if header_type == self.get_header_type():
             return
 
@@ -240,16 +247,27 @@ class ChatListRow(Gtk.ListBoxRow):
         else:
             self.set_header(self._conversations_header)
 
+    def clear(self) -> None:
+        self.timestamp = 0
+        self.stanza_id = None
+        self.message_id = None
+        self.set_message_text('')
+        self.set_nick('')
+
     def set_message_id(self, message_id: str) -> None:
         self.message_id = message_id
 
     def set_message_text(self,
                          text: str,
-                         nickname: Optional[str] = None,
-                         icon_name: Optional[str] = None,
-                         additional_data: Optional[AdditionalDataDict] = None
+                         nickname: str | None = None,
+                         icon_name: str | None = None,
+                         additional_data: AdditionalDataDict | None = None
                          ) -> None:
 
+        assert isinstance(
+            self.contact,
+            BareContact | GroupchatContact | GroupchatParticipant)
+
         draft = app.storage.drafts.get(self.contact)
         if draft is not None:
             self._show_draft(draft)
@@ -313,10 +331,16 @@ class ChatListRow(Gtk.ListBoxRow):
 
     def update_avatar(self) -> None:
         scale = self.get_scale_factor()
+        assert isinstance(
+            self.contact,
+            BareContact | GroupchatContact | GroupchatParticipant)
         surface = self.contact.get_avatar(AvatarSize.ROSTER, scale)
         self._ui.avatar_image.set_from_surface(surface)
 
     def update_name(self) -> None:
+        assert isinstance(
+            self.contact,
+            BareContact | GroupchatContact | GroupchatParticipant)
         if self.type == 'pm':
             client = app.get_client(self.account)
             muc_name = get_groupchat_name(client, self.jid.new_as_bare())
@@ -344,12 +368,12 @@ class ChatListRow(Gtk.ListBoxRow):
             self.message_id,
             self.timestamp)
 
-        if self.contact.is_groupchat:
-            needs_highlight = message_needs_highlight(
-                text,
-                self.contact.nickname,
-                self._client.get_own_jid().bare)
-            if needs_highlight:
+        if (isinstance(self.contact, GroupchatContact) and
+                self.contact.nickname is not None):
+            if message_needs_highlight(
+                    text,
+                    self.contact.nickname,
+                    self._client.get_own_jid().bare):
                 self._needs_muc_highlight = True
                 self._ui.unread_label.get_style_context().remove_class(
                     'unread-counter-silent')
@@ -365,7 +389,8 @@ class ChatListRow(Gtk.ListBoxRow):
         app.storage.cache.reset_unread_count(self.account, self.jid)
 
         # Add class again in case we were mentioned previously
-        if self.contact.is_groupchat and not self.contact.can_notify():
+        if (isinstance(self.contact, GroupchatContact) and
+                not self.contact.can_notify()):
             self._ui.unread_label.get_style_context().add_class(
                 'unread-counter-silent')
 
@@ -437,14 +462,14 @@ class ChatListRow(Gtk.ListBoxRow):
                                    ) -> None:
 
         if event.button == Gdk.BUTTON_SECONDARY:
-            self._popup_menu(event)
+            self._raise_context_popover(event)
 
         elif event.button == Gdk.BUTTON_MIDDLE:
             app.window.activate_action(
                 'remove-chat',
                 GLib.Variant('as', [self.account, str(self.jid)]))
 
-    def _popup_menu(self, event: Gdk.EventButton):
+    def _raise_context_popover(self, event: Gdk.EventButton):
         menu = get_chat_list_row_menu(
             self.workspace_id, self.account, self.jid, self._pinned)
 
@@ -460,8 +485,13 @@ class ChatListRow(Gtk.ListBoxRow):
 
         popover = GajimPopover(menu, relative_to=self)
         popover.set_pointing_to_coord(x=x, y=event.y)
+        popover.connect('closed', self._on_context_popover_closed)
+        self.emit('context-menu-state-changed', True)
         popover.popup()
 
+    def _on_context_popover_closed(self, _popover: Gtk.Popover) -> None:
+        self.emit('context-menu-state-changed', False)
+
     def _on_drag_begin(self,
                        row: ChatListRow,
                        drag_context: Gdk.DragContext
@@ -507,6 +537,15 @@ class ChatListRow(Gtk.ListBoxRow):
 
         elif isinstance(self.contact, GroupchatContact):
             self.contact.connect('avatar-update', self._on_avatar_update)
+            self.contact.connect('disco-info-update',
+                                  self._on_disco_info_update)
+            self.contact.connect('state-changed', self._on_muc_state_changed)
+            self.contact.connect('mam-sync-started', self._on_mam_sync_changed)
+            self.contact.connect('mam-sync-finished', self._on_mam_sync_changed)
+            self.contact.connect('mam-sync-error', self._on_mam_sync_error)
+
+            self._client.connect_signal('state-changed',
+                                        self._on_client_state_changed)
 
         elif isinstance(self.contact, GroupchatParticipant):
             self.contact.connect('chatstate-update', self._on_chatstate_update)
@@ -535,6 +574,78 @@ class ChatListRow(Gtk.ListBoxRow):
                           ) -> None:
         self.update_avatar()
 
+    def _on_disco_info_update(self,
+                              _contact: ChatContactT,
+                              _signal_name: str
+                              ) -> None:
+        self.update_name()
+
+    def _on_client_state_changed(self, *args: Any) -> None:
+        self._update_joined_state()
+
+    def _on_muc_state_changed(self, *args: Any) -> None:
+        self._update_joined_state()
+
+    def _update_joined_state(self) -> None:
+        self._reset_connection_icon()
+        context = self._ui.connection_icon.get_style_context()
+
+        assert isinstance(self.contact, GroupchatContact)
+        if self.contact.is_joining:
+            self._ui.connection_icon.set_from_icon_name(
+                'feather-refresh-cw-symbolic', Gtk.IconSize.MENU)
+            context.add_class('spin')
+            context.add_class('dim-label')
+            self._ui.connection_icon.set_tooltip_text(_('Joining Group Chat…'))
+            self._ui.connection_icon.show()
+        elif (self.contact.is_not_joined or
+                not self._client.state.is_available):
+            self._ui.connection_icon.set_from_icon_name(
+                'feather-zap-symbolic', Gtk.IconSize.MENU)
+            context.add_class('warning-color')
+            self._ui.connection_icon.set_tooltip_text(_('Not connected'))
+            self._ui.connection_icon.show()
+
+    def _on_mam_sync_changed(self,
+                             _contact: GroupchatContact,
+                             signal_name: str
+                             ) -> None:
+
+        self._reset_connection_icon()
+        context = self._ui.connection_icon.get_style_context()
+
+        if signal_name == 'mam-sync-started':
+            self._ui.connection_icon.set_from_icon_name(
+                'feather-refresh-cw-symbolic', Gtk.IconSize.MENU)
+            context.add_class('spin')
+            context.add_class('info-color')
+            self._ui.connection_icon.set_tooltip_text(_('Fetching messages…'))
+            self._ui.connection_icon.show()
+
+    def _on_mam_sync_error(self,
+                           _contact: GroupchatContact,
+                           _signal_name: str,
+                           error_text: str
+                           ) -> None:
+
+        self._reset_connection_icon()
+        self._ui.connection_icon.set_from_icon_name(
+            'feather-zap-symbolic', Gtk.IconSize.MENU)
+        self._ui.connection_icon.get_style_context().add_class('error-color')
+        self._ui.connection_icon.set_tooltip_text(
+            _('There has been an error while trying to '
+              'fetch messages: %s') % error_text)
+        self._ui.connection_icon.show()
+
+    def _reset_connection_icon(self) -> None:
+        self._ui.connection_icon.hide()
+        context = self._ui.connection_icon.get_style_context()
+        context.remove_class('spin')
+        context.remove_class('dim-label')
+        context.remove_class('info-color')
+        context.remove_class('warning-color')
+        context.remove_class('error-color')
+
     def _on_muc_user_update(self,
                             _contact: GroupchatParticipant,
                             _signal_name: str,
