@@ -14,9 +14,6 @@
 
 from __future__ import annotations
 
-from typing import Optional
-from typing import Union
-
 import logging
 import sys
 import time
@@ -26,9 +23,7 @@ from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
-from nbxmpp.errors import StanzaError
 from nbxmpp.protocol import JID
-from nbxmpp.structs import MessageProperties
 
 from gajim.common import app
 from gajim.common import events
@@ -46,19 +41,19 @@ from gajim.common.structs import OutgoingMessage
 from gajim.common.types import ChatContactT
 from gajim.common.util.text import remove_invalid_xml_chars
 
-from .chat_banner import ChatBanner
-from .chat_function_page import ChatFunctionPage
-from .chat_function_page import FunctionMode
-from .const import TARGET_TYPE_URI_LIST
-from .control import ChatControl
-from .dialogs import ErrorDialog
-from .message_actions_box import MessageActionsBox
-from .message_input import MessageInputTextView
-from .util import EventHelper
-from .util import open_window
-from .util import set_urgency_hint
+from gajim.gtk.chat_banner import ChatBanner
+from gajim.gtk.chat_function_page import ChatFunctionPage
+from gajim.gtk.chat_function_page import FunctionMode
+from gajim.gtk.const import TARGET_TYPE_URI_LIST
+from gajim.gtk.control import ChatControl
+from gajim.gtk.dialogs import ErrorDialog
+from gajim.gtk.message_actions_box import MessageActionsBox
+from gajim.gtk.message_input import MessageInputTextView
+from gajim.gtk.util import EventHelper
+from gajim.gtk.util import open_window
+from gajim.gtk.util import set_urgency_hint
 
-log = logging.getLogger('gajim.gui.chatstack')
+log = logging.getLogger('gajim.gtk.chatstack')
 
 
 class ChatStack(Gtk.Stack, EventHelper):
@@ -69,7 +64,7 @@ class ChatStack(Gtk.Stack, EventHelper):
         self.set_vexpand(True)
         self.set_hexpand(True)
 
-        self._current_contact: Optional[ChatContactT] = None
+        self._current_contact: ChatContactT | None = None
 
         self.add_named(ChatPlaceholderBox(), 'empty')
 
@@ -125,8 +120,13 @@ class ChatStack(Gtk.Stack, EventHelper):
         overlay.connect('drag-motion', self._on_drag_motion)
         overlay.connect('drag-leave', self._on_drag_leave)
 
+        if app.is_flatpak():
+            target = 'application/vnd.portal.files'
+        else:
+            target = 'text/uri-list'
+
         uri_entry = Gtk.TargetEntry.new(
-            'text/uri-list',
+            target,
             Gtk.TargetFlags.OTHER_APP,
             TARGET_TYPE_URI_LIST)
         dnd_list = [uri_entry,
@@ -189,7 +189,10 @@ class ChatStack(Gtk.Stack, EventHelper):
             self._current_contact.disconnect_all_from_obj(self)
 
         client = app.get_client(account)
-        self._current_contact = client.get_module('Contacts').get_contact(jid)
+        contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
+        self._current_contact = contact
 
         app.preview_manager.clear_previews()
 
@@ -222,7 +225,7 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         if isinstance(self._current_contact, GroupchatContact):
             muc_data = client.get_module('MUC').get_muc_data(
-                str(self._current_contact.jid))
+                self._current_contact.jid)
             if muc_data is not None:
                 if muc_data.state.is_captcha_request:
                     self._show_chat_function_page(FunctionMode.CAPTCHA_REQUEST)
@@ -261,16 +264,14 @@ class ChatStack(Gtk.Stack, EventHelper):
 
     def _on_room_password_required(self,
                                    _contact: GroupchatContact,
-                                   _signal_name: str,
-                                   _properties: MessageProperties
+                                   _signal_name: str
                                    ) -> None:
 
         self._show_chat_function_page(FunctionMode.PASSWORD_REQUEST)
 
     def _on_room_captcha_challenge(self,
                                    contact: GroupchatContact,
-                                   _signal_name: str,
-                                   _properties: MessageProperties
+                                   _signal_name: str
                                    ) -> None:
 
         self._show_chat_function_page(FunctionMode.CAPTCHA_REQUEST)
@@ -278,39 +279,34 @@ class ChatStack(Gtk.Stack, EventHelper):
     def _on_room_captcha_error(self,
                                _contact: GroupchatContact,
                                _signal_name: str,
-                               error: StanzaError
+                               error: str
                                ) -> None:
 
-        error_text = helpers.to_user_string(error)
-        self._show_chat_function_page(FunctionMode.CAPTCHA_ERROR, error_text)
+        self._show_chat_function_page(FunctionMode.CAPTCHA_ERROR, error)
 
     def _on_room_creation_failed(self,
                                  _contact: GroupchatContact,
                                  _signal_name: str,
-                                 properties: MessageProperties
+                                 error: str
                                  ) -> None:
 
-        assert properties.error is not None
-        error_text = helpers.to_user_string(properties.error)
-        self._show_chat_function_page(FunctionMode.CREATION_FAILED, error_text)
+        self._show_chat_function_page(FunctionMode.CREATION_FAILED, error)
 
     def _on_room_join_failed(self,
                              _contact: GroupchatContact,
                              _signal_name: str,
-                             error: StanzaError
+                             error: str
                              ) -> None:
 
-        self._show_chat_function_page(
-            FunctionMode.JOIN_FAILED, helpers.to_user_string(error))
+        self._show_chat_function_page(FunctionMode.JOIN_FAILED, error)
 
     def _on_room_config_failed(self,
                                _contact: GroupchatContact,
                                _signal_name: str,
-                               error: StanzaError
+                               error: str
                                ) -> None:
 
-        self._show_chat_function_page(
-            FunctionMode.CONFIG_FAILED, helpers.to_user_string(error))
+        self._show_chat_function_page(FunctionMode.CONFIG_FAILED)
 
     def _on_muc_state_changed(self,
                               contact: GroupchatContact,
@@ -353,10 +349,10 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         self._update_group_chat_actions(self._current_contact)
 
-    def _on_account_state(self,
-                          event: Union[events.AccountConnected,
-                                       events.AccountDisconnected]
-                          ) -> None:
+    def _on_account_state(
+        self,
+        event: events.AccountConnected | events.AccountDisconnected
+    ) -> None:
 
         if self._current_contact is None:
             return
@@ -377,6 +373,16 @@ class ChatStack(Gtk.Stack, EventHelper):
         if not event.msgtxt or event.properties.is_sent_carbon:
             return
 
+        client = app.get_client(event.account)
+        contact = client.get_module('Contacts').get_contact(event.jid)
+        if isinstance(contact, GroupchatContact):
+            # MUC messages may be received after some delay, so make sure we
+            # don't issue notifications for our own messages.
+            self_contact = contact.get_self()
+            if (self_contact is not None and
+                    self_contact.name == event.properties.muc_nickname):
+                return
+
         if app.window.is_chat_active(event.account, event.jid):
             return
 
@@ -404,7 +410,7 @@ class ChatStack(Gtk.Stack, EventHelper):
                 _icon, file_type = preview_helpers.guess_simple_file_type(text)
                 text = f'{file_type} ({file_name})'
 
-        sound: Optional[str] = None
+        sound: str | None = None
         msg_type = 'chat-message'
         if isinstance(contact, BareContact):
             msg_type = 'chat-message'
@@ -438,12 +444,16 @@ class ChatStack(Gtk.Stack, EventHelper):
         if tim is not None and time.mktime(time.localtime()) - tim > 1:
             sound = None
 
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
         if app.settings.get('notification_preview_message'):
             if text.startswith('/me') or text.startswith('/me\n'):
                 name = contact.name
                 if isinstance(contact, GroupchatContact):
                     name = event.properties.muc_nickname
                 text = f'* {name} {text[3:]}'
+        else:
+            text = ''
 
         app.ged.raise_event(
             events.Notification(account=contact.account,
@@ -499,7 +509,6 @@ class ChatStack(Gtk.Stack, EventHelper):
             jingle.get_enabled() or
             httpupload.get_enabled())
 
-        app.window.get_action('show-contact-info').set_enabled(online)
         app.window.get_action('correct-message').set_enabled(online)
 
     def _update_chat_actions(self, contact: BareContact) -> None:
@@ -507,10 +516,10 @@ class ChatStack(Gtk.Stack, EventHelper):
         online = app.account_is_connected(account)
 
         app.window.get_action('start-voice-call').set_enabled(
-            online and contact.supports_audio() and
+            online and contact.supports_audio and
             sys.platform != 'win32')
         app.window.get_action('start-video-call').set_enabled(
-            online and contact.supports_video() and
+            online and contact.supports_video and
             sys.platform != 'win32')
 
         app.window.get_action('quote').set_enabled(online)
@@ -547,7 +556,7 @@ class ChatStack(Gtk.Stack, EventHelper):
 
     def _on_action(self,
                    action: Gio.SimpleAction,
-                   param: Optional[GLib.Variant]) -> None:
+                   param: GLib.Variant | None) -> None:
 
         action_name = action.get_name()
         contact = self._current_contact
@@ -670,19 +679,28 @@ class ChatStack(Gtk.Stack, EventHelper):
 
     def _on_drag_motion(self,
                         _widget: Gtk.Widget,
-                        _context: Gdk.DragContext,
+                        context: Gdk.DragContext,
                         _x_coord: int,
                         _y_coord: int,
-                        _time: int
+                        time: int
                         ) -> bool:
-        self._drop_area.set_no_show_all(False)
-        self._drop_area.show_all()
-        return True
+
+        if time == 0:
+            # Workaround for https://gitlab.gnome.org/GNOME/gtk/-/issues/5518
+            return False
+
+        for targ in context.list_targets():
+            if targ.name() in {'text/uri-list', 'application/vnd.portal.files'}:
+                self._drop_area.set_no_show_all(False)
+                self._drop_area.show_all()
+                return True
+
+        return False
 
     def _show_chat_function_page(self,
                                  function_mode: FunctionMode,
-                                 data: Optional[str] = None,
-                                 files: Optional[list[str]] = None
+                                 data: str | None = None,
+                                 files: list[str] | None = None
                                  ) -> None:
 
         assert self._current_contact is not None
@@ -720,8 +738,14 @@ class ChatStack(Gtk.Stack, EventHelper):
         contact = self._current_contact
         assert contact is not None
 
+        client = app.get_client(contact.account)
+
         encryption = contact.settings.get('encryption')
-        if encryption:
+        if encryption == 'OMEMO':
+            if not client.get_module('OMEMO').check_send_preconditions(contact):
+                return
+
+        elif encryption:
             if encryption not in app.plugin_manager.encryption_plugins:
                 ErrorDialog(_('Encryption error'),
                             _('Missing necessary encryption plugin'))
@@ -734,8 +758,6 @@ class ChatStack(Gtk.Stack, EventHelper):
             if not self._chat_control.sendmessage:
                 return
 
-        client = app.get_client(contact.account)
-
         message = remove_invalid_xml_chars(message)
         if message in ('', '\n'):
             return
@@ -770,7 +792,7 @@ class ChatStack(Gtk.Stack, EventHelper):
         self._message_action_box.msg_textview.clear()
         app.storage.drafts.set(contact, '')
 
-    def get_last_message_id(self, contact: ChatContactT) -> Optional[str]:
+    def get_last_message_id(self, contact: ChatContactT) -> str | None:
         return self._message_action_box.get_last_message_id(contact)
 
     def _close_control(self) -> None:
