@@ -17,21 +17,21 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
 from typing import NamedTuple
-from typing import Optional
-from typing import Union
 
+from collections.abc import Callable
+from collections.abc import Iterator
 from enum import Enum
 from enum import IntEnum
 from enum import unique
 
+from gajim.common.i18n import _
 from gajim.common.setting_values import AllSettingsT
 
 
 class Filter(NamedTuple):
     name: str
-    pattern: Union[str, list[str]]
+    pattern: str | list[str]
     default: bool
 
 
@@ -39,15 +39,15 @@ class Setting(NamedTuple):
     kind: SettingKind
     label: str
     type: SettingType
-    value: Optional[AllSettingsT] = None
-    name: Optional[str] = None
-    callback: Optional[Callable[..., None]] = None
-    data: Optional[Any] = None
-    desc: Optional[str] = None
-    bind: Optional[str] = None
-    inverted: Optional[bool] = None
-    enabled_func: Optional[Callable[..., bool]] = None
-    props: Optional[dict[str, Any]] = None
+    value: AllSettingsT | None = None
+    name: str | None = None
+    callback: Callable[..., None] | None = None
+    data: Any | None = None
+    desc: str | None = None
+    bind: str | None = None
+    inverted: bool | None = None
+    enabled_func: Callable[..., bool] | None = None
+    props: dict[str, Any] | None = None
 
 
 DEFAULT_WORKSPACE_COLOR = 'rgb(191,15,167)'
@@ -55,6 +55,30 @@ DEFAULT_WORKSPACE_COLOR = 'rgb(191,15,167)'
 # Drag and drop target type URI list (for dropped files)
 TARGET_TYPE_URI_LIST = 80
 
+MAX_MESSAGE_LENGTH = 5000
+
+
+@unique
+class MuteState(IntEnum):
+    UNMUTE = 0
+    MIN_30 = 30
+    MIN_60 = 60
+    MIN_120 = 120
+    MIN_480 = 480
+    PERM = 100000000
+
+    @classmethod
+    def iter(cls) -> Iterator[tuple[int, str]]:
+        yield from cls._labels.items()  # pyright: ignore
+
+MuteState._labels = {  # pyright: ignore
+        MuteState.MIN_30: _('30 minutes'),
+        MuteState.MIN_60: _('1 hour'),
+        MuteState.MIN_120: _('2 hours'),
+        MuteState.MIN_480: _('8 hours'),
+        MuteState.PERM: _('Permanently'),
+    }
+
 
 @unique
 class Theme(IntEnum):
@@ -118,42 +142,42 @@ class ControlType(Enum):
 
 
 WINDOW_MODULES = {
-    'AccountsWindow': 'gajim.gui.accounts',
-    'AccountWizard': 'gajim.gui.account_wizard',
-    'AddContact': 'gajim.gui.add_contact',
-    'AdHocCommands': 'gajim.gui.adhoc',
-    'AdvancedConfig': 'gajim.gui.advanced_config',
-    'BlockingList': 'gajim.gui.blocking',
-    'Bookmarks': 'gajim.gui.bookmarks',
-    'CertificateDialog': 'gajim.gui.certificate_dialog',
-    'ChangePassword': 'gajim.gui.change_password',
-    'ContactInfo': 'gajim.gui.contact_info',
-    'CreateGroupchatWindow': 'gajim.gui.groupchat_creation',
-    'Features': 'gajim.gui.features',
-    'GroupchatDetails': 'gajim.gui.groupchat_details',
-    'GroupChatInvitation': 'gajim.gui.groupchat_invitation',
-    'GroupchatJoin': 'gajim.gui.groupchat_join',
-    'HistoryExport': 'gajim.gui.history_export',
-    'HistorySyncAssistant': 'gajim.gui.history_sync',
-    'MamPreferences': 'gajim.gui.mam_preferences',
-    'ManageProxies': 'gajim.gui.proxies',
-    'ManageSounds': 'gajim.gui.manage_sounds',
-    'PasswordDialog': 'gajim.gui.password_dialog',
-    'PEPConfig': 'gajim.gui.pep_config',
-    'PluginsWindow': 'gajim.gui.plugins',
-    'Preferences': 'gajim.gui.preferences',
-    'ProfileWindow': 'gajim.gui.profile',
-    'RemoveAccount': 'gajim.gui.remove_account',
-    'RosterItemExchange': 'gajim.gui.roster_item_exchange',
-    'ServerInfo': 'gajim.gui.server_info',
-    'ServiceDiscoveryWindow': 'gajim.gui.discovery',
-    'ServiceRegistration': 'gajim.gui.service_registration',
-    'SSLErrorDialog': 'gajim.gui.ssl_error_dialog',
-    'StartChatDialog': 'gajim.gui.start_chat',
-    'SynchronizeAccounts': 'gajim.gui.synchronize_accounts',
-    'Themes': 'gajim.gui.themes',
-    'WorkspaceDialog': 'gajim.gui.workspace_dialog',
-    'XMLConsoleWindow': 'gajim.gui.xml_console',
+    'AccountsWindow': 'gajim.gtk.accounts',
+    'AccountWizard': 'gajim.gtk.account_wizard',
+    'AddContact': 'gajim.gtk.add_contact',
+    'AdHocCommands': 'gajim.gtk.adhoc',
+    'AdvancedConfig': 'gajim.gtk.advanced_config',
+    'BlockingList': 'gajim.gtk.blocking',
+    'Bookmarks': 'gajim.gtk.bookmarks',
+    'CertificateDialog': 'gajim.gtk.certificate_dialog',
+    'ChangePassword': 'gajim.gtk.change_password',
+    'ContactInfo': 'gajim.gtk.contact_info',
+    'CreateGroupchatWindow': 'gajim.gtk.groupchat_creation',
+    'Features': 'gajim.gtk.features',
+    'GroupchatDetails': 'gajim.gtk.groupchat_details',
+    'GroupChatInvitation': 'gajim.gtk.groupchat_invitation',
+    'GroupchatJoin': 'gajim.gtk.groupchat_join',
+    'HistoryExport': 'gajim.gtk.history_export',
+    'HistorySyncAssistant': 'gajim.gtk.history_sync',
+    'MamPreferences': 'gajim.gtk.mam_preferences',
+    'ManageProxies': 'gajim.gtk.proxies',
+    'ManageSounds': 'gajim.gtk.manage_sounds',
+    'PasswordDialog': 'gajim.gtk.password_dialog',
+    'PEPConfig': 'gajim.gtk.pep_config',
+    'PluginsWindow': 'gajim.gtk.plugins',
+    'Preferences': 'gajim.gtk.preferences',
+    'ProfileWindow': 'gajim.gtk.profile',
+    'RemoveAccount': 'gajim.gtk.remove_account',
+    'RosterItemExchange': 'gajim.gtk.roster_item_exchange',
+    'ServerInfo': 'gajim.gtk.server_info',
+    'ServiceDiscoveryWindow': 'gajim.gtk.discovery',
+    'ServiceRegistration': 'gajim.gtk.service_registration',
+    'SSLErrorDialog': 'gajim.gtk.ssl_error_dialog',
+    'StartChatDialog': 'gajim.gtk.start_chat',
+    'SynchronizeAccounts': 'gajim.gtk.synchronize_accounts',
+    'Themes': 'gajim.gtk.themes',
+    'WorkspaceDialog': 'gajim.gtk.workspace_dialog',
+    'XMLConsoleWindow': 'gajim.gtk.xml_console',
 }
 
 
@@ -166,14 +190,13 @@ APP_ACTIONS = [
     ('copy-text', 's'),
     ('create-groupchat', 's'),
     ('faq', None),
+    ('privacy-policy', None),
     ('features', None),
     ('file-transfer', None),
     ('forget-groupchat', 'a{sv}'),
-    ('groupchat-join', 'as'),
     ('join-support-chat', None),
     ('manage-proxies', None),
     ('open-link', 'as'),
-    ('open-mail', 's'),
     ('plugins', None),
     ('preferences', None),
     ('quit', None),
@@ -181,6 +204,8 @@ APP_ACTIONS = [
     ('shortcuts', None),
     ('show', None),
     ('start-chat', 'as'),
+    ('open-chat', 'as'),
+    ('mute-chat', 'a{sv}'),
     ('xml-console', None),
 ]
 
@@ -193,6 +218,7 @@ MAIN_WIN_ACTIONS = [
     ('input-clear', None, True),
     ('show-emoji-chooser', None, True),
     ('activate-message-selection', 'u', True),
+    ('delete-message-locally', 'u', True),
     ('correct-message', None, False),
     ('copy-message', 's', True),
     ('retract-message', 'a{sv}', False),
@@ -204,7 +230,7 @@ MAIN_WIN_ACTIONS = [
     ('add-to-roster', None, True),
     ('start-voice-call', None, False),
     ('start-video-call', None, False),
-    ('show-contact-info', None, False),
+    ('show-contact-info', None, True),
     ('send-message', None, False),
     ('muc-change-nickname', None, False),
     ('muc-invite', None, False),
@@ -275,7 +301,6 @@ ACCOUNT_ACTIONS = [
     ('export-history', 's'),
     ('import-contacts', 's'),
     ('mark-as-read', 'a{sv}'),
-    ('open-chat', 'as'),
     ('open-event', 'a{sv}'),
     ('pep-config', 's'),
     ('profile', 's'),
@@ -300,7 +325,6 @@ ONLINE_ACCOUNT_ACTIONS = {
     'modify-gateway',
     'bookmarks',
     'import-contacts',
-    'open-chat',
     'pep-config',
     'profile',
     'server-info',
