@@ -16,7 +16,6 @@ from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Optional
 
 import logging
 from pathlib import Path
@@ -41,6 +40,7 @@ from gajim.common.preview_helpers import guess_mime_type
 from gajim.common.preview_helpers import pixbuf_from_data
 
 from gajim.gtk.builder import get_builder
+from gajim.gtk.const import TARGET_TYPE_URI_LIST
 from gajim.gtk.filechoosers import FileChooserDialog
 from gajim.gtk.resource_selector import ResourceSelector
 
@@ -57,7 +57,7 @@ class FileTransferSelector(Gtk.Box):
 
     def __init__(self,
                  contact: types.ChatContactT,
-                 method: Optional[str] = None
+                 method: str | None = None
                  ) -> None:
 
         Gtk.Box.__init__(self)
@@ -66,7 +66,7 @@ class FileTransferSelector(Gtk.Box):
         self._method = method or app.window.get_preferred_ft_method(contact)
 
         client = app.get_client(contact.account)
-        self._max_file_size = client.get_module('HTTPUpload').max_file_size or 0
+        self._max_http_file_size = client.get_module('HTTPUpload').max_file_size
 
         self._ui = get_builder('file_transfer_selector.ui')
         self.add(self._ui.stack)
@@ -90,8 +90,13 @@ class FileTransferSelector(Gtk.Box):
                   'Choose the device you would like to send the '
                   'files to.') % self._contact.name)
 
+        if app.is_flatpak():
+            target = 'application/vnd.portal.files'
+        else:
+            target = 'text/uri-list'
+
         uri_entry = Gtk.TargetEntry.new(
-            'text/uri-list', Gtk.TargetFlags.OTHER_APP, 80)
+            target, Gtk.TargetFlags.OTHER_APP, TARGET_TYPE_URI_LIST)
         dst_targets = Gtk.TargetList.new([uri_entry])
 
         self._ui.listbox.drag_dest_set(
@@ -104,6 +109,11 @@ class FileTransferSelector(Gtk.Box):
 
         self.show_all()
 
+    def _is_over_max_http_file_size(self, path: Path) -> bool:
+        if self._max_http_file_size is None:
+            return False
+        return path.stat().st_size > self._max_http_file_size
+
     def transfer_resource_required(self) -> bool:
         if not isinstance(self._contact, BareContact):
             # No jingle file transfer for group chats
@@ -116,7 +126,7 @@ class FileTransferSelector(Gtk.Box):
         current_page = self._ui.stack.get_visible_child_name()
         if self._method == 'httpupload':
             for path in self._get_file_paths():
-                if path.stat().st_size > self._max_file_size:
+                if self._is_over_max_http_file_size(path):
                     if current_page == 'resource-selection':
                         return False
 
@@ -139,7 +149,7 @@ class FileTransferSelector(Gtk.Box):
         file_paths = self._get_file_paths()
         for path in file_paths:
             if (self._method == 'jingle' or
-                    path.stat().st_size > self._max_file_size):
+                    self._is_over_max_http_file_size(path)):
                 assert self._resource_selector is not None
                 item = (path, 'jingle', self._resource_selector.get_jid())
             elif self._method == 'httpupload':
@@ -162,7 +172,7 @@ class FileTransferSelector(Gtk.Box):
                 continue
 
             size_warning = bool(self._method == 'httpupload' and
-                                path.stat().st_size > self._max_file_size)
+                                self._is_over_max_http_file_size(path))
             jingle_warning = bool(self._method == 'jingle')
 
             row = FileRow(path, size_warning, jingle_warning)
@@ -196,7 +206,7 @@ class FileTransferSelector(Gtk.Box):
         if not selection.get_data():
             return
 
-        if target_type == 80:
+        if target_type == TARGET_TYPE_URI_LIST:
             self.add_files(selection.get_uris())
 
     def _on_files_changed(self, _listbox: Gtk.ListBox, _row: FileRow) -> None:
@@ -208,7 +218,7 @@ class FileTransferSelector(Gtk.Box):
         if self._method == 'httpupload' and self._contact.is_groupchat:
             # Enforce HTTPUpload file size limit for group chats
             for path in file_paths:
-                if path.stat().st_size > self._max_file_size:
+                if self._is_over_max_http_file_size(path):
                     self.emit('changed', False)
                     return
 
@@ -277,8 +287,8 @@ class FileRow(Gtk.ListBoxRow):
         load_file_async(self.file_path, self._on_load_finished, mime_type)
 
     def _on_load_finished(self,
-                          data: Optional[bytes],
-                          _error: Optional[GLib.Error],
+                          data: bytes | None,
+                          _error: GLib.Error | None,
                           mime_type: Any
                           ) -> None:
 
