@@ -14,34 +14,33 @@
 
 from __future__ import annotations
 
-from typing import Optional
-
 from gi.repository import Gdk
 from gi.repository import GObject
 from gi.repository import Gtk
 
 from gajim.common import app
-from gajim.common import ged
+from gajim.common import types
 from gajim.common.const import AvatarSize
-from gajim.common.events import MucDiscoUpdate
+from gajim.common.const import SimpleClientState
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import GroupchatContact
-from gajim.gtk.groupchat_affiliation import GroupchatAffiliation
-from gajim.gtk.groupchat_outcasts import GroupchatOutcasts
 
-from .builder import get_builder
-from .groupchat_config import GroupchatConfig
-from .groupchat_info import GroupChatInfoScrolled
-from .groupchat_manage import GroupchatManage
-from .groupchat_settings import GroupChatSettings
-from .sidebar_switcher import SideBarSwitcher
-from .structs import RemoveHistoryActionParams
+from gajim.gtk.builder import get_builder
+from gajim.gtk.groupchat_affiliation import GroupchatAffiliation
+from gajim.gtk.groupchat_config import GroupchatConfig
+from gajim.gtk.groupchat_info import GroupChatInfoScrolled
+from gajim.gtk.groupchat_manage import GroupchatManage
+from gajim.gtk.groupchat_outcasts import GroupchatOutcasts
+from gajim.gtk.groupchat_settings import GroupChatSettings
+from gajim.gtk.omemo_trust_manager import OMEMOTrustManager
+from gajim.gtk.sidebar_switcher import SideBarSwitcher
+from gajim.gtk.structs import RemoveHistoryActionParams
 
 
 class GroupchatDetails(Gtk.ApplicationWindow):
     def __init__(self,
                  contact: GroupchatContact,
-                 page: Optional[str] = None
+                 page: str | None = None
                  ) -> None:
         Gtk.ApplicationWindow.__init__(self)
         self.set_application(app.app)
@@ -54,27 +53,36 @@ class GroupchatDetails(Gtk.ApplicationWindow):
 
         self.account = contact.account
         self._client = app.get_client(contact.account)
+        self._client.connect_signal(
+            'state-changed', self._on_client_state_changed)
+
         self._contact = contact
         self._contact.connect('avatar-update', self._on_avatar_update)
+        self._contact.connect('disco-info-update', self._on_disco_info_update)
 
         self._ui = get_builder('groupchat_details.ui')
         self._ui.connect_signals(self)
 
         self._switcher = SideBarSwitcher(width=250)
-        self._switcher.set_stack(self._ui.main_stack)
+        self._switcher.set_stack(self._ui.main_stack, rows_visible=False)
         self._ui.main_grid.attach(self._switcher, 0, 0, 1, 1)
         self._ui.main_stack.connect('notify::visible-child-name',
                                     self._on_stack_child_changed)
         self.add(self._ui.main_grid)
 
-        self._groupchat_manage: Optional[GroupchatManage] = None
+        self._groupchat_manage: GroupchatManage | None = None
 
         self._add_groupchat_info()
         self._add_groupchat_settings()
-        self._add_groupchat_manage()
-        self._add_affiliations()
-        self._add_outcasts()
-        self._add_configuration()
+        self._add_groupchat_encryption()
+
+        if self._client.state.is_available:
+            self._ui.edit_name_button.set_sensitive(True)
+            self._ui.edit_name_button.set_tooltip_text(_('Edit Name…'))
+            self._add_groupchat_manage()
+            self._add_affiliations()
+            self._add_outcasts()
+            self._add_configuration()
 
         self._load_avatar()
         self._ui.name_entry.set_text(contact.name)
@@ -82,17 +90,26 @@ class GroupchatDetails(Gtk.ApplicationWindow):
         if page is not None:
             self._switcher.set_row(page)
 
-        app.ged.register_event_handler(
-            'muc-disco-update', ged.GUI1, self._on_muc_disco_update)
-
         self.connect('key-press-event', self._on_key_press)
         self.connect('destroy', self._on_destroy)
 
         self.show_all()
 
-    def _on_muc_disco_update(self, event: MucDiscoUpdate) -> None:
-        if event.jid != self._contact.jid:
-            return
+    def _on_client_state_changed(self,
+                                 _client: types.Client,
+                                 _signal_name: str,
+                                 state: SimpleClientState
+                                 ) -> None:
+
+        self._ui.edit_name_button.set_sensitive(state.is_connected)
+        self._ui.edit_name_button.set_tooltip_text(
+            _('Not connected') if not state.is_connected else _('Edit Name…'))
+
+    def _on_disco_info_update(self,
+                              _contact: GroupchatContact,
+                              _signal_name: str
+                              ) -> None:
+
         self._ui.name_entry.set_text(self._contact.name)
         disco_info = self._contact.get_disco()
         assert disco_info is not None
@@ -136,6 +153,7 @@ class GroupchatDetails(Gtk.ApplicationWindow):
         self._groupchat_manage = GroupchatManage(self.account,
                                                  self._contact)
         self._ui.manage_box.add(self._groupchat_manage)
+        self._switcher.set_row_visible('manage', True)
 
     def _add_groupchat_info(self) -> None:
         self._groupchat_info = GroupChatInfoScrolled(
@@ -146,6 +164,7 @@ class GroupchatDetails(Gtk.ApplicationWindow):
         self._groupchat_info.set_from_disco_info(disco_info)
         self._groupchat_info.set_subject(self._contact.subject)
         self._ui.info_box.add(self._groupchat_info)
+        self._switcher.set_row_visible('information', True)
 
     def _add_groupchat_settings(self) -> None:
         main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=24)
@@ -172,18 +191,33 @@ class GroupchatDetails(Gtk.ApplicationWindow):
         scrolled_window.add(main_box)
 
         self._ui.settings_box.add(scrolled_window)
+        self._switcher.set_row_visible('settings', True)
+
+    def _add_groupchat_encryption(self) -> None:
+        if (self._contact.is_groupchat and
+                self._contact.muc_context == 'public'):
+            # OMEMO is not available for public group chats
+            self._switcher.set_row_visible('encryption-omemo', False)
+            return
+
+        self._ui.encryption_box.add(
+            OMEMOTrustManager(self._contact.account, self._contact))
+        self._switcher.set_row_visible('encryption-omemo', True)
 
     def _add_affiliations(self) -> None:
         affiliations = GroupchatAffiliation(self._client, self._contact)
         self._ui.affiliation_box.add(affiliations)
+        self._switcher.set_row_visible('affiliations', True)
 
     def _add_outcasts(self) -> None:
         affiliations = GroupchatOutcasts(self._client, self._contact)
         self._ui.outcasts_box.add(affiliations)
+        self._switcher.set_row_visible('outcasts', True)
 
     def _add_configuration(self) -> None:
         config = GroupchatConfig(self._client, self._contact)
         self._ui.configuration_box.add(config)
+        self._switcher.set_row_visible('config', True)
 
     def _on_key_press(self,
                       _widget: GroupchatDetails,
@@ -192,7 +226,4 @@ class GroupchatDetails(Gtk.ApplicationWindow):
             self.destroy()
 
     def _on_destroy(self, _widget: GroupchatDetails) -> None:
-        app.ged.remove_event_handler('muc-disco-update',
-                                     ged.GUI1,
-                                     self._on_muc_disco_update)
         app.check_finalize(self)
