@@ -14,19 +14,15 @@
 
 from __future__ import annotations
 
-from typing import Optional
-from typing import Tuple
-
 from gi.repository import Gtk
+
 try:
     from gi.repository import Gst
 except Exception:
     pass
 
 
-def create_gtk_widget() -> Optional[Tuple[Gst.Element,
-                                          Gtk.Widget,
-                                          str]]:
+def create_gtk_widget() -> tuple[Gst.Element, Gtk.Widget, str] | None:
     gtkglsink = Gst.ElementFactory.make('gtkglsink', None)
     if gtkglsink is not None:
         glsinkbin = Gst.ElementFactory.make('glsinkbin', None)
