@@ -15,7 +15,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import logging
 import tempfile
@@ -36,6 +35,7 @@ from gajim.common.commands import CommandFailed
 from gajim.common.const import Direction
 from gajim.common.const import SimpleClientState
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.types import ChatContactT
@@ -44,12 +44,9 @@ from gajim.gtk.builder import get_builder
 from gajim.gtk.dialogs import ErrorDialog
 from gajim.gtk.menus import get_encryption_menu
 from gajim.gtk.menus import get_format_menu
-from gajim.gtk.menus import get_groupchat_menu
-from gajim.gtk.menus import get_private_chat_menu
-from gajim.gtk.menus import get_self_contact_menu
-from gajim.gtk.menus import get_singlechat_menu
 from gajim.gtk.message_input import MessageInputTextView
 from gajim.gtk.security_label_selector import SecurityLabelSelector
+from gajim.gtk.util import open_window
 
 log = logging.getLogger('gajim.gtk.messageactionsbox')
 
@@ -58,8 +55,8 @@ class MessageActionsBox(Gtk.Grid):
     def __init__(self) -> None:
         Gtk.Grid.__init__(self)
 
-        self._client: Optional[Client] = None
-        self._contact: Optional[ChatContactT] = None
+        self._client: Client | None = None
+        self._contact: ChatContactT | None = None
 
         self._ui = get_builder('message_actions_box.ui')
         self.get_style_context().add_class('message-actions-box')
@@ -93,8 +90,6 @@ class MessageActionsBox(Gtk.Grid):
         self._ui.formattings_button.set_menu_model(get_format_menu())
         self._ui.encryption_menu_button.set_menu_model(get_encryption_menu())
 
-        self._ui.quick_invite_button.set_action_name('win.muc-invite')
-
         self.show_all()
         self._ui.connect_signals(self)
 
@@ -107,7 +102,7 @@ class MessageActionsBox(Gtk.Grid):
         assert self._contact is not None
         return self._contact
 
-    def get_seclabel(self) -> Optional[SecurityLabel]:
+    def get_seclabel(self) -> SecurityLabel | None:
         return self._security_label_selector.get_seclabel()
 
     def _connect_actions(self) -> None:
@@ -137,7 +132,7 @@ class MessageActionsBox(Gtk.Grid):
 
     def _on_action(self,
                    action: Gio.SimpleAction,
-                   param: Optional[GLib.Variant]) -> Optional[int]:
+                   param: GLib.Variant | None) -> int | None:
 
         if self._contact is None:
             return
@@ -181,26 +176,21 @@ class MessageActionsBox(Gtk.Grid):
         self._contact = contact
 
         if isinstance(self._contact, GroupchatContact):
-            self._ui.quick_invite_button.show()
             self._contact.multi_connect({
                 'state-changed': self._on_muc_state_changed,
                 'user-role-changed': self._on_muc_state_changed,
             })
         elif isinstance(self._contact, GroupchatParticipant):
-            self._ui.quick_invite_button.hide()
             self._contact.multi_connect({
                 'user-joined': self._on_user_state_changed,
                 'user-left': self._on_user_state_changed,
                 'room-joined': self._on_user_state_changed,
                 'room-left': self._on_user_state_changed,
             })
-        else:
-            self._ui.quick_invite_button.hide()
-
-        self._set_settings_menu(contact)
 
         self._update_encryption_button()
         self._update_encryption_details_button()
+        self._update_send_file_button_tooltip()
 
         self._set_chatstate(True)
 
@@ -227,10 +217,10 @@ class MessageActionsBox(Gtk.Grid):
     def toggle_message_correction(self) -> None:
         self.msg_textview.toggle_message_correction()
 
-    def try_message_correction(self, message: str) -> Optional[str]:
+    def try_message_correction(self, message: str) -> str | None:
         return self.msg_textview.try_message_correction(message)
 
-    def get_last_message_id(self, contact: ChatContactT) -> Optional[str]:
+    def get_last_message_id(self, contact: ChatContactT) -> str | None:
         return self.msg_textview.get_last_message_id(contact)
 
     def _on_client_state_changed(self,
@@ -290,7 +280,7 @@ class MessageActionsBox(Gtk.Grid):
         if current_state == new_state:
             return
 
-        if new_state:
+        if new_state and new_state != 'OMEMO':
             plugin = app.plugin_manager.encryption_plugins.get(new_state)
             if plugin is None:
                 # TODO: Add GUI error here
@@ -327,6 +317,12 @@ class MessageActionsBox(Gtk.Grid):
                             'Encryption is not available.')
                 icon_name = 'channel-insecure-symbolic'
 
+                # Disable encryption if chat was encrypted before, but due to
+                # changed circumstances, encryption is not applicable anymore
+                # (i.e. group chat configuration changed).
+                contact.settings.set('encryption', '')
+                action.set_state(GLib.Variant('s', ''))
+
         self._ui.encryption_menu_button.set_sensitive(sensitive)
         self._ui.encryption_menu_button.set_tooltip_text(tooltip)
         self._ui.encryption_image.set_from_icon_name(
@@ -334,17 +330,21 @@ class MessageActionsBox(Gtk.Grid):
 
     def _update_encryption_details_button(self) -> None:
         contact = self.get_current_contact()
-        state = contact.settings.get('encryption')
+        encryption = contact.settings.get('encryption')
 
-        encryption_state = {'visible': bool(state),
-                            'enc_type': state,
+        encryption_state = {'visible': bool(encryption),
+                            'enc_type': encryption,
                             'authenticated': False}
 
-        if state:
-            app.plugin_manager.extension_point(
-                f'encryption_state{state}',
-                app.window.get_control(),
-                encryption_state)
+        if encryption:
+            if encryption == 'OMEMO':
+                encryption_state['authenticated'] = True
+            else:
+                # Only fire extension_point for plugins (i.e. not OMEMO)
+                app.plugin_manager.extension_point(
+                    f'encryption_state{encryption}',
+                    app.window.get_control(),
+                    encryption_state)
 
         visible, enc_type, authenticated = encryption_state.values()
         assert isinstance(visible, bool)
@@ -371,20 +371,27 @@ class MessageActionsBox(Gtk.Grid):
     def _on_encryption_details_clicked(self, _button: Gtk.Button) -> None:
         contact = self.get_current_contact()
         encryption = contact.settings.get('encryption')
+        if encryption == 'OMEMO':
+            if contact.is_groupchat:
+                open_window('GroupchatDetails',
+                            contact=contact,
+                            page='encryption-omemo')
+                return
+
+            if isinstance(contact, BareContact) and contact.is_self:
+                window = open_window('AccountsWindow')
+                window.select_account(contact.account, page='encryption-omemo')
+                return
+
+            open_window('ContactInfo',
+                        account=contact.account,
+                        contact=contact,
+                        page='encryption-omemo')
+            return
+
         app.plugin_manager.extension_point(
             f'encryption_dialog{encryption}', app.window.get_control())
 
-    def _set_settings_menu(self, contact: ChatContactT) -> None:
-        if isinstance(contact, GroupchatContact):
-            menu = get_groupchat_menu(contact)
-        elif isinstance(contact, GroupchatParticipant):
-            menu = get_private_chat_menu(contact)
-        elif contact.is_self:
-            menu = get_self_contact_menu(contact)
-        else:
-            menu = get_singlechat_menu(contact)
-        self._ui.settings_menu.set_menu_model(menu)
-
     def _on_format(self, name: str) -> None:
         name = name.removeprefix('input-')
         self.msg_textview.apply_formatting(name)
@@ -420,7 +427,8 @@ class MessageActionsBox(Gtk.Grid):
                     units = GLib.FormatSizeFlags.IEC_UNITS
                 else:
                     units = GLib.FormatSizeFlags.DEFAULT
-                max_file_size = GLib.format_size_full(max_file_size, units)
+                max_file_size = GLib.format_size_full(
+                    int(max_file_size), units)
                 tooltip_text = _('Send File (max. %s)…') % max_file_size
 
         self._ui.sendfile_button.set_tooltip_text(tooltip_text)
