@@ -27,8 +27,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
-from typing import Union
 
 import logging
 import sys
@@ -52,6 +50,8 @@ from gajim.common.ged import EventHelper
 from gajim.common.helpers import allow_showing_notification
 from gajim.common.helpers import play_sound
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatParticipant
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.structs import AccountJidParam
@@ -83,7 +83,8 @@ class NotificationBackend(EventHelper):
 
         self.register_events([
             ('notification', ged.GUI2, self._on_notification),
-            ('account-enabled', ged.GUI2, self._on_account_enabled)
+            ('account-enabled', ged.GUI2, self._on_account_enabled),
+            ('chat-read', ged.GUI2, self._on_chat_read)
         ])
 
         for client in app.get_clients():
@@ -91,6 +92,14 @@ class NotificationBackend(EventHelper):
                                   self._on_client_state_changed)
 
     def _on_notification(self, event: events.Notification) -> None:
+        if event.account and event.jid:
+            client = app.get_client(event.account)
+            contact = client.get_module('Contacts').get_contact(event.jid)
+
+            if contact.is_muted:
+                log.debug('Notifications muted for %s', contact)
+                return
+
         if event.sound is not None:
             play_sound(event.sound, event.account)
 
@@ -103,6 +112,9 @@ class NotificationBackend(EventHelper):
         client = app.get_client(event.account)
         client.connect_signal('state-changed', self._on_client_state_changed)
 
+    def _on_chat_read(self, event: events.ChatRead) -> None:
+        self._withdraw(['new-message', event.account, event.jid])
+
     def _on_client_state_changed(self,
                                  client: Client,
                                  _signal_name: str,
@@ -171,7 +183,7 @@ class PopupNotification(Gtk.Window):
         self.set_decorated(False)
         self.set_keep_above(True)
 
-        self._timeout_id: Optional[int] = None
+        self._timeout_id: int | None = None
         self._event = event
 
         self._ui = get_builder('popup_notification_window.ui')
@@ -241,7 +253,7 @@ class PopupNotification(Gtk.Window):
                          _widget: Gtk.Widget,
                          event: Gdk.EventButton
                          ) -> None:
-        if event.button == 1:
+        if event.button == Gdk.BUTTON_PRIMARY:
 
             jid = ''
             if self._event.jid is not None:
@@ -372,7 +384,7 @@ class Linux(NotificationBackend):
                 _('Mark as Read'), action, params.to_variant())
 
     def _make_notification_id(self,
-                              event: events.Notification) -> Optional[str]:
+                              event: events.Notification) -> str | None:
         if event.type in ('connection-failed', 'server-shutdown'):
             return self._make_id([event.type, event.account])
 
@@ -407,11 +419,12 @@ class Linux(NotificationBackend):
 
 
 def _get_avatar_for_notification(account: str,
-                                 jid: Union[JID, str]) -> GdkPixbuf.Pixbuf:
+                                 jid: JID | str) -> GdkPixbuf.Pixbuf:
     scale = get_monitor_scale_factor()
     size = AvatarSize.NOTIFICATION
     client = app.get_client(account)
     contact = client.get_module('Contacts').get_contact(jid)
+    assert isinstance(contact, BareContact | GroupchatParticipant)
     surface = contact.get_avatar(size, scale)
     pixbuf = Gdk.pixbuf_get_from_surface(surface,
                                          0,
