@@ -15,8 +15,6 @@
 
 from typing import Any
 from typing import cast
-from typing import Optional
-from typing import Union
 
 import logging
 
@@ -32,9 +30,12 @@ from nbxmpp.namespaces import Namespace
 from nbxmpp.task import Task
 
 from gajim.common import app
+from gajim.common.client import Client
 from gajim.common.const import AvatarSize
+from gajim.common.const import SimpleClientState
 from gajim.common.i18n import _
 from gajim.common.i18n import p_
+from gajim.common.modules.contacts import BareContact
 
 from gajim.gtk.avatar import clip_circle
 from gajim.gtk.avatar_selector import AvatarSelector
@@ -79,6 +80,9 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._jid = app.get_jid_from_account(account)
 
         self._client = app.get_client(self.account)
+        self._client.connect_signal(
+            'state-changed', self._on_client_state_changed)
+
         self._contact = self._client.get_module('Contacts').get_contact(
             self._jid)
 
@@ -91,15 +95,15 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._ui.add_entry_button.set_menu_model(menu)
         self._add_actions()
 
-        self._avatar_selector: Optional[AvatarSelector] = None
-        self._current_avatar: Optional[cairo.ImageSurface] = None
-        self._current_vcard: Optional[VCard] = None
-        self._avatar_nick_public: Optional[bool] = None
+        self._avatar_selector: AvatarSelector | None = None
+        self._current_avatar: cairo.ImageSurface | None = None
+        self._current_vcard: VCard | None = None
+        self._avatar_nick_public: bool | None = None
 
         # False  - no change to avatar
         # None   - we want to delete the avatar
         # Avatar - upload new avatar
-        self._new_avatar: Union[None, bool, Avatar] = False
+        self._new_avatar: None | bool | Avatar = False
 
         self._ui.nickname_entry.set_text(app.nicks[account])
 
@@ -145,6 +149,16 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._ui.privacy_popover.destroy()
         app.check_finalize(self)
 
+    def _on_client_state_changed(self,
+                                 _client: Client,
+                                 _signal_name: str,
+                                 state: SimpleClientState
+                                 ) -> None:
+
+        self._ui.save_button.set_sensitive(state.is_connected)
+        self._ui.save_button.set_tooltip_text(
+            _('Not connected') if not state.is_connected else '')
+
     def _on_access_model_received(self, task: Task) -> None:
         namespace = task.get_user_data()
 
@@ -170,9 +184,12 @@ class ProfileWindow(Gtk.ApplicationWindow):
 
     def _on_vcard_received(self, task: Task):
         try:
-            self._current_vcard = cast(VCard, task.finish())
+            self._current_vcard = cast(VCard | None, task.finish())
         except StanzaError as error:
             log.info('Error loading VCard: %s', error)
+            self._current_vcard = None
+
+        if self._current_vcard is None:
             self._current_vcard = VCard()
 
         self._load_avatar()
@@ -182,6 +199,7 @@ class ProfileWindow(Gtk.ApplicationWindow):
 
     def _load_avatar(self) -> None:
         scale = self.get_scale_factor()
+        assert isinstance(self._contact, BareContact)
         self._current_avatar = self._contact.get_avatar(AvatarSize.VCARD,
                                                         scale,
                                                         add_show=False)
@@ -201,7 +219,7 @@ class ProfileWindow(Gtk.ApplicationWindow):
 
     def _on_action(self,
                    action: Gio.SimpleAction,
-                   _param: Optional[GLib.Variant]
+                   _param: GLib.Variant | None
                    ) -> None:
         name = action.get_name()
         key = name.split('-')[1]
@@ -272,6 +290,8 @@ class ProfileWindow(Gtk.ApplicationWindow):
                 public=public,
                 callback=self._on_set_avatar)
 
+        self._avatar_nick_public = public
+
         nick = GLib.markup_escape_text(self._ui.nickname_entry.get_text())
         client.get_module('UserNickname').set_nickname(nick, public=public)
 
@@ -304,6 +324,7 @@ class ProfileWindow(Gtk.ApplicationWindow):
 
     def _on_remove_avatar(self, _button: Gtk.Button) -> None:
         scale = self.get_scale_factor()
+        assert isinstance(self._contact, BareContact)
         surface = self._contact.get_avatar(AvatarSize.VCARD,
                                            scale,
                                            add_show=False,
