@@ -20,7 +20,6 @@
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 from typing import Any
-from typing import Union
 
 import logging
 import sys
@@ -40,9 +39,9 @@ from gajim.common.helpers import get_global_show
 from gajim.common.helpers import get_uf_show
 from gajim.common.i18n import _
 
-from .builder import get_builder
-from .util import get_icon_name
-from .util import open_window
+from gajim.gtk.builder import get_builder
+from gajim.gtk.util import get_icon_name
+from gajim.gtk.util import open_window
 
 if app.is_installed('APPINDICATOR'):
     # pylint: disable=ungrouped-imports
@@ -53,7 +52,7 @@ elif app.is_installed('AYATANA_APPINDICATOR'):
     from gi.repository import AyatanaAppIndicator3 as AppIndicator
 
 
-log = logging.getLogger('gajim.gui.statusicon')
+log = logging.getLogger('gajim.gtk.statusicon')
 
 
 class StatusIcon:
@@ -170,10 +169,10 @@ class GtkMenuBackend(EventHelper):
     def _on_our_show(self, _event: events.ShowChanged) -> None:
         self.update_state()
 
-    def _on_account_state(self,
-                          _event: Union[events.AccountConnected,
-                                        events.AccountDisconnected]
-                          ) -> None:
+    def _on_account_state(
+        self,
+        _event: events.AccountConnected | events.AccountDisconnected
+    ) -> None:
         account_connected = bool(app.get_number_of_connected_accounts() > 0)
         self._ui.start_chat_menuitem.set_sensitive(account_connected)
 
@@ -303,7 +302,8 @@ class AppIndicatorIcon(GtkMenuBackend):
             'Gajim',
             'org.gajim.Gajim',
             AppIndicator.IndicatorCategory.COMMUNICATIONS)
-        self._status_icon.set_icon_theme_path(str(configpaths.get('ICONS')))
+        if not app.is_flatpak():
+            self._status_icon.set_icon_theme_path(str(configpaths.get('ICONS')))
         self._status_icon.set_status(AppIndicator.IndicatorStatus.ACTIVE)
         self._status_icon.set_menu(self._ui.systray_context_menu)
         self._status_icon.set_secondary_activate_target(
