@@ -18,10 +18,11 @@ from gi.repository import Gtk
 from gajim.common import app
 from gajim.common.exceptions import GajimGeneralException
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
 
-from .dialogs import ErrorDialog
-from .util import get_app_window
-from .builder import get_builder
+from gajim.gtk.builder import get_builder
+from gajim.gtk.dialogs import ErrorDialog
+from gajim.gtk.util import get_app_window
 
 
 class SynchronizeAccounts(Gtk.ApplicationWindow):
@@ -100,10 +101,8 @@ class SynchronizeAccounts(Gtk.ApplicationWindow):
             iter_ = model.append()
             model.set(
                 iter_,
-                0,
-                remote_account,
-                1,
-                app.get_hostname_from_account(remote_account))
+                {0: remote_account,
+                 1: app.get_hostname_from_account(remote_account)})
 
     def _on_next_clicked(self, _button: Gtk.Button) -> None:
         selection = self._ui.accounts_treeview.get_selection()
@@ -146,7 +145,10 @@ class SynchronizeAccounts(Gtk.ApplicationWindow):
         for remote_jid in remote_jid_list:
             if remote_jid not in local_jid_list:
                 iter_ = model.append()
-                model.set(iter_, 0, True, 1, remote_jid)
+                model.set(
+                    iter_,
+                    {0: True,
+                     1: remote_jid})
 
     def _on_sync_toggled(self, cell: Gtk.CellRendererToggle, path: str) -> None:
         model = self._ui.contacts_treeview.get_model()
@@ -170,6 +172,7 @@ class SynchronizeAccounts(Gtk.ApplicationWindow):
                             'contact list?' % hostname)
                 remote_contact = self._remote_client.get_module(
                     'Contacts').get_contact(remote_jid)
+                assert isinstance(remote_contact, BareContact)
 
                 # keep same groups and same nickname
                 self._local_client.get_module('Presence').subscribe(
