@@ -15,42 +15,39 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
-from typing import Type
 
+import logging
 import os
 import sys
-import json
 import zipfile
-import logging
-from shutil import rmtree, move
-from dataclasses import dataclass
-from pathlib import Path
-from importlib.util import spec_from_file_location
+from collections.abc import Callable
 from importlib.util import module_from_spec
+from importlib.util import spec_from_file_location
+from pathlib import Path
+from shutil import move
+from shutil import rmtree
+from types import TracebackType
 
-from packaging.version import Version as V
-from packaging.requirements import Requirement
-
-import gajim
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import modules
 from gajim.common.client import Client
-from gajim.common.events import PluginAdded
-from gajim.common.events import PluginRemoved
-from gajim.common.i18n import _
 from gajim.common.exceptions import PluginsystemError
-from gajim.common.helpers import Singleton
-from gajim.plugins.plugins_i18n import _ as p_
+from gajim.common.i18n import _
+from gajim.common.util.classes import Singleton
 
-from gajim.plugins.helpers import GajimPluginActivateException
-from gajim.plugins.helpers import is_shipped_plugin
-from gajim.plugins.gajimplugin import GajimPlugin
-from gajim.plugins.gajimplugin import GajimPluginException
+from .events import PluginAdded
+from .events import PluginRemoved
+from .gajimplugin import GajimPlugin
+from .gajimplugin import GajimPluginException
+from .helpers import GajimPluginActivateException
+from .helpers import is_shipped_plugin
+from .manifest import PluginManifest
 
 log = logging.getLogger('gajim.p.manager')
 
+RmErrorT = tuple[type[BaseException], BaseException, TracebackType]
+
 FIELDS = ('name',
           'short_name',
           'version',
@@ -58,80 +55,6 @@ FIELDS = ('name',
           'authors',
           'homepage')
 
-GAJIM_VERSION = V(gajim.__version__.split('+', 1)[0]).base_version
-
-
-@dataclass(frozen=True, eq=True)
-class PluginManifest:
-    name: str
-    short_name: str
-    description: str
-    authors: list[str]
-    homepage: str
-    config_dialog: bool
-    version: V
-    requirements: list[Requirement]
-    platforms: list[str]
-    path: Optional[Path] = None
-
-    def __hash__(self):
-        return hash(f'{self.short_name}_{self.version}')
-
-    @property
-    def is_usable(self) -> bool:
-        if not self._check_requirements():
-            return False
-        return sys.platform in self.platforms
-
-    @property
-    def is_shipped(self) -> bool:
-        if self.path is None:
-            return False
-        return is_shipped_plugin(self.path)
-
-    def get_remote_url(self, repository_url: str) -> str:
-        filename = f'{self.short_name}_{self.version}.zip'
-        return f'{repository_url}/{self.short_name}/{filename}'
-
-    @classmethod
-    def from_path(cls, path: Path) -> PluginManifest:
-        manifest_path = path / 'plugin-manifest.json'
-        if not manifest_path.exists():
-            raise ValueError(f'Not a plugin path: {path}')
-
-        if manifest_path.is_dir():
-            raise ValueError(f'Not a plugin path: {path}')
-
-        with manifest_path.open(encoding='utf8') as f:
-            try:
-                manifest = json.load(f)
-            except Exception as error:
-                raise ValueError(f'Error while parsing manifest: '
-                                 f'{path}, {error}')
-        return cls.from_manifest_json(manifest, manifest_path)
-
-    def _check_requirements(self) -> bool:
-        for req in self.requirements:
-            if GAJIM_VERSION in req.specifier:
-                return True
-        return False
-
-    @classmethod
-    def from_manifest_json(cls,
-                           manifest: dict[str, Any],
-                           path: Optional[Path] = None) -> PluginManifest:
-        return cls(
-            name=manifest['name'],
-            short_name=manifest['short_name'],
-            description=p_(manifest['description']),
-            authors=manifest['authors'],
-            homepage=manifest['homepage'],
-            config_dialog=manifest['config_dialog'],
-            version=V(manifest['version']),
-            requirements=[Requirement(r) for r in manifest['requirements']],
-            platforms=manifest['platforms'],
-            path=path.parent if path is not None else path)
-
 
 class PluginManager(metaclass=Singleton):
     '''
@@ -173,7 +96,7 @@ class PluginManager(metaclass=Singleton):
     def update_plugins(self,
                        replace: bool = True,
                        activate: bool = False,
-                       plugin_name: Optional[str] = None
+                       plugin_name: str | None = None
                        ) -> list[str]:
         '''
         Move plugins from the downloaded folder to the user plugin folder
@@ -227,7 +150,7 @@ class PluginManager(metaclass=Singleton):
 
     def _load_plugin_module(self,
                             manifest: PluginManifest
-                            ) -> Optional[Type[GajimPlugin]]:
+                            ) -> type[GajimPlugin] | None:
 
         assert manifest.path is not None
         module_path = manifest.path / '__init__.py'
@@ -252,15 +175,15 @@ class PluginManager(metaclass=Singleton):
         for module_attr_name in dir(module):
             module_attr = getattr(module, module_attr_name)
             if issubclass(module_attr, GajimPlugin):
-                setattr(module_attr, 'manifest', manifest)
-                setattr(module_attr, '__path__', str(manifest.path))
+                module_attr.manifest = manifest
+                module_attr.__path__ = str(manifest.path)
                 return module_attr
         return None
 
     def add_plugin(self,
                    manifest: PluginManifest,
                    activate: bool = False
-                   ) -> Optional[GajimPlugin]:
+                   ) -> GajimPlugin | None:
 
         plugin_class = self._load_plugin_module(manifest)
         if plugin_class is None:
@@ -310,7 +233,7 @@ class PluginManager(metaclass=Singleton):
         # name to avoid name problems (removing module_abc if base_package is
         # module_ab)
         modules_to_remove = [module for module in sys.modules
-                             if module.startswith('{}.'.format(base_package))]
+                             if module.startswith(f'{base_package}.')]
         # remove the base_package itself
         if base_package in sys.modules:
             modules_to_remove.append(base_package)
@@ -318,13 +241,13 @@ class PluginManager(metaclass=Singleton):
         for module_to_remove in modules_to_remove:
             del sys.modules[module_to_remove]
 
-    def get_active_plugin(self, plugin_name: str) -> Optional[GajimPlugin]:
+    def get_active_plugin(self, plugin_name: str) -> GajimPlugin | None:
         for plugin in self.active_plugins:
             if plugin.manifest.short_name == plugin_name:
                 return plugin
         return None
 
-    def get_plugin(self, short_name: str) -> Optional[GajimPlugin]:
+    def get_plugin(self, short_name: str) -> GajimPlugin | None:
         for plugin in self.plugins:
             if plugin.manifest.short_name == short_name:
                 return plugin
@@ -433,8 +356,8 @@ class PluginManager(metaclass=Singleton):
         :type args: tuple
 
         '''
-        if ((gui_extpoint_name not in self.gui_extension_points)
-                or (args not in self.gui_extension_points[gui_extpoint_name])):
+        if (gui_extpoint_name not in self.gui_extension_points or
+                args not in self.gui_extension_points[gui_extpoint_name]):
             self.gui_extension_points.setdefault(gui_extpoint_name, []).append(
                 args)
 
@@ -474,7 +397,7 @@ class PluginManager(metaclass=Singleton):
     def _register_modules_with_handlers(self, plugin: GajimPlugin) -> None:
         if not hasattr(plugin, 'modules'):
             return
-        for client in app.connections.values():
+        for client in app.get_clients():
             for module in plugin.modules:
                 instance, name = module.get_instance(client)
                 modules.register_single_module(client, instance, name)
@@ -485,7 +408,7 @@ class PluginManager(metaclass=Singleton):
     def _unregister_modules_with_handlers(self, plugin: GajimPlugin) -> None:
         if not hasattr(plugin, 'modules'):
             return
-        for client in app.connections.values():
+        for client in app.get_clients():
             for module in plugin.modules:
                 instance = client.get_module(module.name)
                 modules.unregister_single_module(client, module.name)
@@ -543,10 +466,12 @@ class PluginManager(metaclass=Singleton):
 
         self._remove_events_handler_from_ged(plugin)
         self._remove_name_from_encryption_plugins(plugin)
-        self._unregister_modules_with_handlers(plugin)
 
-        # removing plug-in from active plug-ins list
+        # deactivate() must be before _unregister_modules_with_handlers(),
+        # because plugin.deactivate() may want to use the module
         plugin.deactivate()
+
+        self._unregister_modules_with_handlers(plugin)
         self.active_plugins.remove(plugin)
         app.settings.set_plugin_setting(plugin.manifest.short_name,
                                         'active',
@@ -601,7 +526,7 @@ class PluginManager(metaclass=Singleton):
                     client.connection.register_handler(handler)
 
     @staticmethod
-    def _load_manifest(plugin_path: Path) -> Optional[PluginManifest]:
+    def _load_manifest(plugin_path: Path) -> PluginManifest | None:
         try:
             return PluginManifest.from_path(plugin_path)
         except Exception as error:
@@ -638,7 +563,7 @@ class PluginManager(metaclass=Singleton):
     def install_from_zip(self,
                          zip_filename: str,
                          overwrite: bool = False
-                         ) -> Optional[GajimPlugin]:
+                         ) -> GajimPlugin | None:
         '''
         Install plugin from zip and return plugin
         '''
@@ -647,7 +572,7 @@ class PluginManager(metaclass=Singleton):
         except zipfile.BadZipfile:
             # it is not zip file
             raise PluginsystemError(_('Archive corrupted'))
-        except IOError:
+        except OSError:
             raise PluginsystemError(_('Archive empty'))
 
         if zip_file.testzip():
@@ -694,7 +619,11 @@ class PluginManager(metaclass=Singleton):
         return self.add_plugin(manifest)
 
     def delete_plugin_files(self, plugin_path: Path) -> None:
-        def on_error(func, path, error):
+        def _on_error(func: Callable[..., Any],
+                      path: Path,
+                      error: RmErrorT
+                      ) -> None:
+
             if func is os.path.islink:
                 # if symlink
                 os.unlink(path)
@@ -702,7 +631,7 @@ class PluginManager(metaclass=Singleton):
             # access is denied or other
             raise PluginsystemError(str(error[1]))
 
-        rmtree(plugin_path, False, on_error)
+        rmtree(plugin_path, False, _on_error)
 
     def uninstall_plugin(self, plugin: GajimPlugin) -> None:
         '''
@@ -726,7 +655,7 @@ class PluginManager(metaclass=Singleton):
 
         app.ged.raise_event(PluginRemoved(manifest=plugin.manifest))
 
-    def get_plugin_by_path(self, plugin_dir: str) -> Optional[GajimPlugin]:
+    def get_plugin_by_path(self, plugin_dir: str) -> GajimPlugin | None:
         for plugin in self.plugins:
             if plugin.__path__ in plugin_dir:
                 return plugin
